// ****************************************************************************
//  Name: example-detach.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 08/08/2000 : mdb - Created.
//
//  Desc: Sample filter application - Pipe-detached, stand alone.
// ****************************************************************************

#include "example-filters.h"

class P : public Producer {};
class F : public PassByFilter {};
class C : public Consumer {};

DC_Filter_Base_t *filter_factory(char *sbFilterName) {
  cout << "app: creating a new object for filter " << sbFilterName << endl;
  if (strcmp(sbFilterName, "P") == 0) {  
    return new P;
  } else if (strcmp(sbFilterName, "F") == 0) {  
    return new F;
  } else if (strcmp(sbFilterName, "C") == 0) {  
    return new C;
  } else {
    cerr << "app: unknown filter \"" << sbFilterName << "\" to create" << endl;
  }
  return NULL;
}

int main(int argc, char *argv[]) {

#ifdef _WIN32
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
 
	wVersionRequested = MAKEWORD(2, 0);
 
	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) {
		cout << "Error, Winsock could not be initialized.";
		return 0;
	}  

#endif /* WIN32 */

  DC_FilterService_t DC;
  if (DC.init("example-detach", &filter_factory, &argc, &argv)) exit(1);

  // emulator args
  Global.wTotalSize    = 4096;
  Global.wNumPiece     = 4;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;

  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }
  // we are the console process
  
  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  //layout.ReadFile("example-detach_filterlayout.cfg");
  layout.Add("P", NULL, "P-F");
  layout.Add("F", "P-F", "F-C");
  layout.Add("C", "F-C", NULL);
  cout << layout << endl;

  // static work, and no client-server front end
  DC_Work_t work;
  work.buf.sprintf("query = 1 100 100 400 400 8");
  work.buf.Append('\0');

  // determine where to place filters
  DC_Placement_t placement;
  if (DC.PlacementPlanning(layout, work, placement) != 0) { 
    // we choose our own placement instead
    placement.Add("P", "<one_per_node>");
        // UVC: it won't work with single appd!
        // placement.Add("F", "<one_per_node>");
    placement.Add("F", "<one_per_node>", 2);
    placement.Add("C", "<one_per_node>");
  }
  cout << "example-detach: " << placement;

  int wMaxBatches      = (argc > 1 ? atoi(argv[1]) : 1);
  int wMaxWorkPerBatch = (argc > 2 ? atoi(argv[2]) : 1);
  int wMaxInstances    = (argc > 3 ? atoi(argv[3]) : 1);

  cout << "example-detach: "
       << wMaxBatches << " batches of "
       << wMaxWorkPerBatch << " uow per batch, each appended to "
       << wMaxInstances << " instances" << endl;

  // get a filter instance (existing or new) for executing work
  int ret;
  DC_FilterInstance_t **rgpinstance = new DC_FilterInstance_t*[wMaxInstances];
  for (int i=0; i<wMaxInstances; ++i) {
    if ((ret = DC.NewFilterInstance(layout, work, placement, rgpinstance[i]))
	!= DC_ERR_OK) {
      cerr << "app: failed GetFilterInstance() rtn=" << ret << endl
	   << DC_strerror(ret) << endl;
    }
  }

  // append work to instances
  int *rgwh = new int[wMaxInstances * wMaxWorkPerBatch];
  if (!rgwh) {
    cerr << "failed to allocate" << endl;
    exit(1);
  }

  for (int j=0; j<wMaxBatches; ++j) {
    for (int i=0; i<wMaxInstances; ++i) {
      for (int k=0; k<wMaxWorkPerBatch; ++k) {
	if ((rgwh[i*wMaxWorkPerBatch+k] = rgpinstance[i]->AppendWork(work)) < 0) {
	  cerr << "app: failed AppendWork() work=" << j << " instance=" << i
	       << " rtn=" << rgwh[i*wMaxWorkPerBatch+k] << endl
	       << DC_strerror(rgwh[i*wMaxWorkPerBatch+k]) << endl;
	}
      }
    }

#if 1
    // wait for this particular set of work to finish
    for (int i=0; i<wMaxInstances; ++i) {
      for (int k=0; k<wMaxWorkPerBatch; ++k) {
	if (rgwh[i*wMaxWorkPerBatch+k] < 0) continue;
	ret = DC.WaitWork(rgwh[i*wMaxWorkPerBatch+k]);
	cout << "app: wh=" << rgwh[i*wMaxWorkPerBatch+k] << " finished, ret="
	     << ret << endl;
      }
    }
#endif
  }

#if 0
  // wait for any of the submitted work to finish
  int wh;
  while ((wh = DC.WaitAnyWork()) != DC_ERR_NoWorkFound) {
    if (wh < 0) {
      cerr << "app: failed WaitAnyWork() call" << DC_strerror(wh) << endl;
    } else {
      cout << "app: wh=" << wh << " finished, ret="
	   << DC.ExitStatus(wh) << endl;
    }
  }
#endif

  delete [] rgwh;
  delete [] rgpinstance;
  return 0;
}


// ****************************************************************************
