// ****************************************************************************
//  Name: example-dlib.cpp
//  Athr: Mike Gray (mgray)
//  Date: 05/20/2002 : mgray - Created.
//
//  Desc: Sample dynamic library filter application - Pipe-detached, stand alone.
// ****************************************************************************

#include <assert.h>

#include <DataCutter.h>
#include <lib/dc_library.h>



DC_Filter_Base_t *filter_factory(char *sbFilterName) {
  DC_Filter_Base_t *(*factoryHandle)(char *str);
  Library *lib = new Library("../example/filters-dlib.so");

  cout << "example-dlib: creating new object for filter " << sbFilterName << endl;

  if (lib->load()) {
    cerr << "error: unable to load library: " << lib->getErrorMsg() << endl;
    lib->unload();
    return NULL;
  }

  factoryHandle = (DC_Filter_Base_t *(*)(char *))lib->getFunction("filter_factory");
  if (NULL == factoryHandle) {
    cerr << "error: unable to find filter_factory in library: " << lib->getErrorMsg() << endl;
    lib->unload();
    return NULL;
  }

  return (*factoryHandle)(sbFilterName);
}


int main(int argc, char *argv[]) {
  DC_FilterService_t DC;

  if (DC.init("example-dlib", &filter_factory, &argc, &argv)) exit(1);

  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }
  // we are the console process

  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  if (layout.ReadFile("example-dlib_layout.cfg")) {
    cerr << "error: unable to read layout file" << endl;
    exit(1);
  }
  cout << "example-dlib: " << layout << endl;

  // static work, and no client-server front end
  DC_Work_t work;
  work.buf.sprintf("nothing");
  work.buf.Append('\0');

  // determine where to place filters
  DC_Placement_t placement;
  if (DC.PlacementPlanning(layout, work, placement) != 0) { 
    if (placement.ReadFile("example-dlib_placement.cfg")) {
      cerr << "error: unable to read placement file" << endl;
      exit(1);
    }
  }
  cout << "example-dlib: " << placement << endl;

  // get a filter instance (existing or new) for executing work
  int ret;
  DC_FilterInstance_t *instance;
  if ((ret = DC.NewFilterInstance(layout, work, placement, instance)) != DC_ERR_OK) {
    cerr << "app: failed GetFilterInstance() rtn=" << ret << endl
	 << DC_strerror(ret) << endl;
  }

  // append work to instances
  if (instance->AppendWork(work) < 0) {
    cerr << "app: failed AppendWork()" << endl;
  }

  // wait for this particular set of work to finish
  DC.WaitAnyWork();
  cout << "example-dlib: finished" << endl;

  return 0;
}

