// ****************************************************************************
//  Name: example-fitlers.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 08/08/2000 : mdb - Created.
//        04/02/2002 : clh - Moved filter definitions to this file
//  Desc: 
// ****************************************************************************
#include <assert.h>
#include <DataCutter.h>

#define DCDBG_READWRITE

struct Global_s {
  int wTotalSize;
  int wNumPiece;
  double rOverheadBuf;
  double rOverheadByte;
} Global;

void touch_buffer(DC_Buffer_t *pbuf) {
  char *p = pbuf->getPtr();
  for (unsigned int i=0; i<pbuf->getSize(); ++i) {
    p[i] += 1;
  }
}

class Producer : public DC_Filter_Base_t {
  int wBufSize;
  DC_Buffer_t buf;

public:
  DC_RTN_t init(initarg_t &initarg) {

    wBufSize = Global.wTotalSize / Global.wNumPiece;
    buf.New(wBufSize);
    memset(buf.getPtr(), 0, wBufSize);
    buf.setSize(wBufSize);
    buf.setConsume(false);  // do this to avoid deleting a static object

    cout << initarg.sbCopyName << ":init(): "
	 << "within=" << initarg.rankWithinCopySet()
	 << '/' << initarg.maxWithinCopySet() << ' '
	 << "copysets=" << initarg.rankCopySets()
	 << '/' << initarg.maxCopySets() << ' '
	 << "copies=" << initarg.rankCopies()
	 << '/' << initarg.maxCopies() << endl;

    return DC_RTN_OK;
  };

  DC_RTN_t process(arg_t &arg) {
    CpuDelay delay(1);
    cout << arg.sbFilterName << "::process() entry, work=\"" 
	 << arg.pwork->buf.getPtr() << "\"" << endl;

    for (int i = 0; i < Global.wNumPiece; ++i) {
      delay.BusyWait(Global.rOverheadBuf + 
		     Global.rOverheadByte * buf.getMax());
      touch_buffer(&buf);
#if defined(DCDBG_READWRITE)
      cout << arg.sbFilterName << "::write " << buf << endl;
#endif /* DCDBG_READWRITE */
      arg.outs[0].write(&buf);
    }
    cout << arg.sbFilterName << "::process() exit." << endl;
    //return DC_RTN_EndOfFilter;
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) { 
    return DC_RTN_OK; 
  };
};

class PassByFilter : public DC_Filter_Base_t {
public:  // DC_Filter_Base_t interface
  DC_RTN_t init(initarg_t &initarg) {
    cout << initarg.sbCopyName << ":init(): "
	 << "within=" << initarg.rankWithinCopySet()
	 << '/' << initarg.maxWithinCopySet() << ' '
	 << "copysets=" << initarg.rankCopySets()
	 << '/' << initarg.maxCopySets() << ' '
	 << "copies=" << initarg.rankCopies()
	 << '/' << initarg.maxCopies() << endl;
    return DC_RTN_OK;
  };

  DC_RTN_t process(arg_t &arg) {
    DC_Buffer_t *pbuf;
    CpuDelay delay(1);
    cout << arg.sbFilterName << "::process() entry, work=\"" 
	 << arg.pwork->buf.getPtr() << "\"" << endl;
#if 1
    for (int i=0; i<Global.wNumPiece; ++i) {
#else
    while (1) {
#endif
      if ((pbuf = arg.ins[0].read()) == NULL) break;
#if defined(DCDBG_READWRITE)
      cout << arg.sbFilterName << "::read " << *pbuf << endl;
#endif /* DCDBG_READWRITE */
      
      delay.BusyWait(Global.rOverheadBuf + 
		     Global.rOverheadByte * pbuf->getMax());
      touch_buffer(pbuf);

#if defined(DCDBG_READWRITE)
      cout << arg.sbFilterName << "::write " << *pbuf << endl;
#endif /* DCDBG_READWRITE */
      arg.outs[0].write_nocopy(pbuf);
    }
    cout << arg.sbFilterName << "::process() exit." << endl;
    //return arg.ins[0].isEndOfWork() ? DC_RTN_EndOfWork : DC_RTN_EndOfFilter;
    return DC_RTN_OK;
  };

    DC_RTN_t finalize(void) {
    return DC_RTN_OK; 
  };
};

class Consumer : public DC_Filter_Base_t {
public:  // DC_Filter_Base_t interface
  DC_RTN_t init(initarg_t &initarg) {
    cout << initarg.sbCopyName << ":init(): "
	 << "within=" << initarg.rankWithinCopySet()
	 << '/' << initarg.maxWithinCopySet() << ' '
	 << "copysets=" << initarg.rankCopySets()
	 << '/' << initarg.maxCopySets() << ' '
	 << "copies=" << initarg.rankCopies()
	 << '/' << initarg.maxCopies() << endl;
    return DC_RTN_OK;
  };

  DC_RTN_t process(arg_t &arg) {
    DC_Buffer_t *pbuf;
    CpuDelay delay(1);
    cout << arg.sbFilterName << "::process() entry, work=\""
         << arg.pwork->buf.getPtr() << "\"" << endl;
    while (1) {
      if ((pbuf = arg.ins[0].read()) == NULL) break;
#if defined(DCDBG_READWRITE)
      cout << arg.sbFilterName << "::read " << *pbuf << endl;
#endif /* DCDBG_READWRITE */

      //sleep(1);
      delay.BusyWait(Global.rOverheadBuf + 
		     Global.rOverheadByte * pbuf->getMax());

      touch_buffer(pbuf);
      pbuf->consume();
    }
    cout << arg.sbFilterName << "::process() exit." << endl;
    //return arg.ins[0].isEndOfWork() ? DC_RTN_EndOfWork : DC_RTN_EndOfFilter;
    return DC_RTN_OK;
  };

  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

class NullFilter : public DC_Filter_Base_t {
public:  // DC_Filter_Base_t interface
  DC_RTN_t init(initarg_t &initarg) { return DC_RTN_OK; };
  DC_RTN_t process(arg_t &arg) {
    cout << arg.sbFilterName << "::process() entry, work=\""
         << arg.pwork->buf.getPtr() << "\"" << endl;

    // do nothing

    cout << arg.sbFilterName << "::process() exit." << endl;
    //return DC_RTN_EndOfFilter;
    return DC_RTN_OK;
  };
  DC_RTN_t finalize(void) { return DC_RTN_OK; };
};

class SpeedProducer : public DC_Filter_Base_t {
  int wBufSize;
  DC_Buffer_t buf;
  double rDelayTime;

public:  // DC_Filter_Base_t interface
  SpeedProducer(double rDelayTime_in) : rDelayTime(rDelayTime_in) {};  // ctor

  DC_RTN_t init(initarg_t &initarg) {
    wBufSize = 1024;
    buf.New(wBufSize);
    memset(buf.getPtr(), 0, wBufSize);
    buf.setSize(wBufSize);
    buf.setConsume(false);  // do this to avoid deleting a static object
    return DC_RTN_OK;
  };

  DC_RTN_t process(arg_t &arg) {
    CpuDelay delay(1);
    cout << arg.sbFilterName << "::process() entry, work=\""
         << arg.pwork->buf.getPtr() << "\"" << endl;

    for (int i=0; i<Global.wNumPiece; ++i) {

      delay.BusyWait(rDelayTime);

#if defined(DCDBG_READWRITE)
      cout << arg.sbFilterName << "::write " << buf << endl;
#endif /* DCDBG_READWRITE */
      arg.outs[0].write(&buf);
    }
    cout << arg.sbFilterName << "::process() exit." << endl;
    //return DC_RTN_EndOfFilter;
    return DC_RTN_OK;
  };
  DC_RTN_t finalize(void) { return DC_RTN_OK; };
};

class SpeedConsumer : public DC_Filter_Base_t {
  double rDelayTime;

public:  // DC_Filter_Base_t interface
  SpeedConsumer(double rDelayTime_in) : rDelayTime(rDelayTime_in) {};  // ctor

  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  };

  DC_RTN_t process(arg_t &arg) {
    DC_Buffer_t *pbuf;
    CpuDelay delay(1);
    cout << arg.sbFilterName << "::process() entry, work=\""
         << arg.pwork->buf.getPtr() << "\"" << endl;
    while (1) {
      if ((pbuf = arg.ins[0].read()) == NULL) break;
#if defined(DCDBG_READWRITE)
      cout << arg.sbFilterName << "::read " << *pbuf << endl;
#endif /* DCDBG_READWRITE */

      delay.BusyWait(rDelayTime);

      pbuf->consume();
    }
    cout << arg.sbFilterName << "::process() exit." << endl;
    //return arg.ins[0].isEndOfWork() ? DC_RTN_EndOfWork : DC_RTN_EndOfFilter;
    return DC_RTN_OK;
  };

  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};
