// ****************************************************************************
//  Name: example-select.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 09/10/1999 : mdb - Created.
//        03/04/2001 : mdb - Updated to current DC api.
//
//  Desc: Sample application stream based user program.
//        Tests the insSelect() call for multiple input streams in a filter.
// ****************************************************************************

#include <DataCutter.h>


void display_buffer_ints(DC_Buffer_t *pbuf) {
  int *p = (int*) pbuf->getPtr();
  int num = pbuf->getSize() / sizeof(int);
  for (int i=0; i<num; i++) {
    cout << p[i] << ' ';
  }
  cout << endl;
}

class producer1 : public DC_Filter_Base_t {
public:  // DC_Filter_Base_t interface
  DC_RTN_t init(initarg_t &) { return DC_RTN_OK; };
  DC_RTN_t process(arg_t &arg) {
    int sum = 0;
    cout << sbFilterName << ": entry ...\n";

    DC_Buffer_t buf(128);
    for (int i=0; i<100; i++) {
      buf.Empty();
      buf.Append(100+i);
      sum += 100+i;

      //sleep(4);
      cout << sbFilterName << ": generate into stream "
	   << arg.outs[0].getName() << " : ";
      display_buffer_ints(&buf);
      arg.outs[0].write(&buf);
    }

    cout << sbFilterName << ": exit, sum=" << sum << "\n";
    return DC_RTN_OK;
  };

  DC_RTN_t finalize(void) { 
    return DC_RTN_OK; 
  };
};

class producer2 : public DC_Filter_Base_t {
public:  // DC_Filter_Base_t interface
  DC_RTN_t init(initarg_t &) { 
    return DC_RTN_OK; 
  };

  DC_RTN_t process(arg_t &arg) {
    int sum = 0;
    cout << sbFilterName << ": entry ...\n";

    DC_Buffer_t buf(128);
    for (int i=0; i<100; i++) {
      buf.Empty();
      buf.Append(200+i);
      sum += 200+i;

      //sleep(2);
      cout << sbFilterName << ": generate into stream "
	   << arg.outs[0].getName() << " : ";
      display_buffer_ints(&buf);
      arg.outs[0].write(&buf);
    }

    cout << sbFilterName << ": exit, sum=" << sum << "\n";
    return DC_RTN_OK;
  };

  DC_RTN_t finalize(void) { 
    return DC_RTN_OK; 
  };
};

class producer3 : public DC_Filter_Base_t {
public:  // DC_Filter_Base_t interface
  DC_RTN_t init(initarg_t &) { return DC_RTN_OK; };
  DC_RTN_t process(arg_t &arg) {
    int sum = 0;
    cout << sbFilterName << ": entry ...\n";

    DC_Buffer_t buf(128);
    for (int i=0; i<100; i++) {
      buf.Empty();
      buf.Append(300+i);
      sum += 300+i;

      //sleep(1);
      cout << sbFilterName << ": generate into stream "
	   << arg.outs[0].getName() << " : ";
      display_buffer_ints(&buf);
      arg.outs[0].write(&buf);
    }

    cout << sbFilterName << ": exit, sum=" << sum << "\n";
    return DC_RTN_OK;
  };
  DC_RTN_t finalize(void) { return DC_RTN_OK; };
};

class consumer : public DC_Filter_Base_t {
public:  // DC_Filter_Base_t interface
  DC_RTN_t init(initarg_t &) { 
    return DC_RTN_OK; 
  };

  DC_RTN_t process(arg_t &arg) {
    int wRtn = DC_RTN_OK;
    DC_Buffer_t *pbuf;
    int total = 0, sum = 0;
    cout << sbFilterName << ": entry ...\n";

    int i;
    while (1) {
      //sleep(2);
      cout << sbFilterName << ": calling insReadAny() ...\n";
      pbuf = arg.insReadAny(&i);
      if (!pbuf) {
	if (arg.insAllEOW()) {
	  cout << sbFilterName << ": read collective EOW from stream "
	       << arg.ins[i].getName() << endl;
	} else {
	  cout << sbFilterName << ": read collective EOS from stream "
	       << arg.ins[i].getName() << endl;
	  //wRtn = DC_RTN_EndOfFilter;
	  wRtn = DC_RTN_OK;
	}
	break;
      }
      cout << sbFilterName << ": read from stream " << arg.ins[i].getName()
	   << " " << *pbuf << endl;

      total++;
      sum += *(int*)pbuf->getPtr();

      display_buffer_ints(pbuf);

      pbuf->consume();
    }

    cout << sbFilterName << ": exit, sum=" << sum
	 << ", after reading " << total << " values.\n";
    cout << "DEBUG: sum should be (100..199)+(200..299)+(300..399) = 74850"
	 << endl;
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) { 
    return DC_RTN_OK; 
  };
};


DC_Filter_Base_t *filter_factory(char *sbFilterName) {
  cout << "App: creating a new object for filter " << sbFilterName << endl;
  if (strcmp(sbFilterName, "producer1") == 0) {
    return new producer1;
  } else if (strcmp(sbFilterName, "producer2") == 0) {
    return new producer2;
  } else if (strcmp(sbFilterName, "producer3") == 0) {
    return new producer3;
  } else if (strcmp(sbFilterName, "consumer") == 0) {
    return new consumer;
  } else {
    cerr << "App: unknown filter \"" << sbFilterName << "\" to create\n";
  }
  return NULL;
}


int main(int argc, char *argv[]) {

#ifdef _WIN32
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
 
	wVersionRequested = MAKEWORD(2, 0);
 
	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) {
		cout << "Error, Winsock could not be initialized.";
		return 0;
	}  

#endif /* WIN32 */

  DC_FilterService_t DC;
  DC.init("example-select", &filter_factory, &argc, &argv);
  if (DC.isRemoteProcess()) { DC.RemoteProcess(); return 0; }
  // we are the console process

  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  layout.Add("producer1", NULL, "s1");
  layout.Add("producer2", NULL, "s2");
  layout.Add("producer3", NULL, "s3");
  layout.Add("consumer", "s1 s2 s3", NULL);
  cout << layout << endl;

  // static work, and no client-server front end
  DC_Work_t work;
  work.buf.sprintf("query = 1 100 100 400 400 8");
  work.buf.Append('\0');

  // determine where to place filters
  DC_Placement_t placement;
  if (DC.PlacementPlanning(layout, work, placement) != 0) { 
    // we choose our own placement instead
    placement.Add("producer1", "<one_per_node>");
    placement.Add("producer2", "<one_per_node>");
    placement.Add("producer3", "<one_per_node>");
    placement.Add("consumer",  "<one_per_node>");
  }

  int wMaxBatches      = (argc > 1 ? atoi(argv[1]) : 1);
  int wMaxWorkPerBatch = (argc > 2 ? atoi(argv[2]) : 1);
  int wMaxInstances    = (argc > 3 ? atoi(argv[3]) : 1);

  cout << "example-select: "
       << wMaxBatches << " batches of "
       << wMaxWorkPerBatch << " uow per batch, each appended to "
       << wMaxInstances << " instances" << endl;

  // get a filter instance (existing or new) for executing work
  int ret;
  DC_FilterInstance_t **rgpinstance = new DC_FilterInstance_t*[wMaxInstances];
  for (int i=0; i<wMaxInstances; ++i) {
    //if ((ret = DC.GetFilterInstance(layout, work, placement, rgpinstance[i]))
    if ((ret = DC.NewFilterInstance(layout, work, placement, rgpinstance[i]))
	!= DC_ERR_OK) {
      cerr << "app: failed GetFilterInstance() rtn=" << ret << endl
	   << DC_strerror(ret) << endl;
    }
  }

  // append work to instances
  int *rgwh = new int[wMaxInstances * wMaxWorkPerBatch];
  if (!rgwh) {
    cerr << "failed to allocate" << endl;
    exit(1);
  }

  for (int j=0; j<wMaxBatches; ++j) {
    for (int i=0; i<wMaxInstances; ++i) {
      for (int k=0; k<wMaxWorkPerBatch; ++k) {
	if ((rgwh[i*k+k] = rgpinstance[i]->AppendWork(work)) < 0) {
	  cerr << "app: failed AppendWork() work=" << j << " instance=" << i
	       << " rtn=" << rgwh[i*k+k] << endl
	       << DC_strerror(rgwh[i*k+k]) << endl;
	}
      }
    }

#if 1
    // wait for this particular set of work to finish
    for (int i=0; i<wMaxInstances; ++i) {
      for (int k=0; k<wMaxWorkPerBatch; ++k) {
	if (rgwh[i*k+k] < 0) continue;
	cout << "app: wh=" << rgwh[i*k+k] << " finished, ret="
	     << DC.WaitWork(rgwh[i*k+k]) << endl;
      }
    }
#endif
  }

#if 0
  // wait for any of the submitted work to finish
  int wh;
  while ((wh = DC.WaitAnyWork()) != DC_ERR_NoWorkFound) {
    if (wh < 0) {
      cerr << "app: failed WaitAnyWork() call" << DC_strerror(wh) << endl;
    } else {
      cout << "app: wh=" << wh << " finished, ret="
	   << DC.ExitStatus(wh) << endl;
    }
  }
#endif

  delete [] rgwh;
  return 0;
}


// ****************************************************************************
