// ****************************************************************************
//  Name: example-test.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 08/08/2000 : mdb - Created.
//
//  Desc: Sample filter application - Pipe-detached, stand alone.
// ****************************************************************************

#include "example-filters.h"

#ifdef _WIN32
#include "win32.h"
#endif

// Rename the filters
class P : public Producer {};
class F : public PassByFilter {};
class C : public Consumer {};
class A : public NullFilter {};
class B : public NullFilter {};

DC_Filter_Base_t *filter_factory(char *sbFilterName) {
  cout << "app: creating a new object for filter " << sbFilterName << endl;

  // test1
  if (strcmp(sbFilterName, "P") == 0) {  return new P;
  } else if (strcmp(sbFilterName, "F") == 0) {  
    return new F;
  } else if (strcmp(sbFilterName, "C") == 0) {  
    return new C;

  // test2
  } else if (strcmp(sbFilterName, "A") == 0) {  
    return new A;
  } else if (strcmp(sbFilterName, "B") == 0) {  
    return new B;

  // test3
  } else if (strcmp(sbFilterName, "P1") == 0) { 
    return new SpeedProducer(1);
  } else if (strcmp(sbFilterName, "P2") == 0) { 
    return new SpeedProducer(1);
  } else if (strcmp(sbFilterName, "C1") == 0) { 
    return new SpeedConsumer(5);
  } else if (strcmp(sbFilterName, "C2") == 0) { 
    return new SpeedConsumer(0);

  } else {
    cerr << "app: unknown filter \"" << sbFilterName << "\" to create" << endl;
  }
  return NULL;
}

// for test4
class MyPolicy : public DC_Policy_Base_t {
public:
  // in: wMax     - total number of transparent copies stream writes to
  //     sbStream - name of output stream
  //     buf      - actual buffer written by the filter
  //     pUserArg - user defined argument (from write() call)
  // rtn: int - 0..wMax-1 copyset to write to
  int write_copyset(int wMax, char *sbStream, DC_Buffer_t *pbuf, void *pUserArg) {
    int wRtn = wMax-1;
    cout << "app: write_copyset for max= " << wMax << " stream " << sbStream
	 << " ret=" << wRtn << " buf=" << *pbuf << endl;
    return wRtn;
  };
};

DC_Policy_Base_t *policy_factory(char *sbPolicyName) {
  cout << "app: creating a new object for policy " << sbPolicyName << endl;
  // test4
  if (strcmp(sbPolicyName, "distr_accum") == 0) { return new MyPolicy;
  } else {
    cerr << "app: unknown policy \"" << sbPolicyName << "\" to create" << endl;
  }
  return NULL;
}

//
// Test 1
//
int do_test1(int argc, char *argv[]) {
  DC_FilterService_t DC;
  if (DC.init("example-test", &filter_factory, &argc, &argv)) exit(1);

  // emulator args
  Global.wTotalSize    = 4096;
  Global.wNumPiece     = 4;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;

  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }
  // we are the console process

  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
#if 0
  layout.ReadFile("example-detach_filterlayout.cfg");
#else
  layout.Add("P", NULL, "P-F");
  layout.Add("F", "P-F", "F-C");
  layout.Add("C", "F-C", NULL);
#endif
  //cout << layout << endl;

  // static work, and no client-server front end
  DC_Work_t work;
  work.buf.sprintf("query = 1 100 100 400 400 8");
  work.buf.Append('\0');

  // determine where to place filters
  DC_Placement_t placement;
  char *sbNodes = DC.FindAppConfig("nodes");
  if (sbNodes) {
    Str str(sbNodes);
    DArray<char*> *pda = str.TokenArrayNoDup();
    assert(pda->NumElem() == 3);
    placement.Add("P", pda->Get(0));
    placement.Add("F", pda->Get(1));
    placement.Add("C", pda->Get(2));
    delete pda;
  } else {
    placement.Add("P", "<one_per_node>");
    placement.Add("F", "<one_per_node>");
    placement.Add("C", "<one_per_node>");
  }


  Random rand;
  int num_instance;
  for (int j = 0; j < 100; ++j) {
    cout << "*** test " << j << endl;

    // create the number of required filter instances
    int i, ret;
    num_instance = rand.GetRandRange(1,20);
    cout << "num_instance= " << num_instance << endl;
    assert(num_instance > 0);
    DC_FilterInstance_t **rgpinstance = new DC_FilterInstance_t*[num_instance];
    for (i = 0; i < num_instance; ++i) {
      cout << "starting instance " << i << " ..." << endl;
      if ((ret = DC.NewFilterInstance(layout, work, placement, rgpinstance[i]))
	  != DC_ERR_OK) {
	cerr << "app: failed NewFilterInstance() rtn=" << ret << endl
	     << DC_strerror(ret) << endl;
	exit(1);
      }
    }

    sleep(1);

    // now just shut them all down
    for (i=0; i<num_instance; ++i) {
      cout << "stopping instance " << i << " ..." << endl;
      DC.StopFilterInstance(rgpinstance[i], true);
    }

    delete [] rgpinstance;
  }

  return 0;
}

//
// Test 2
//
int do_test2(int argc, char *argv[]) {
  DC_FilterService_t DC;
  if (DC.init("example-test", &filter_factory, &argc, &argv)) exit(1);

  // emulator args
  Global.wTotalSize    = 4096;
  Global.wNumPiece     = 4;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;

  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }
  // we are the console process

  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  layout.Add("A", NULL, "S");
  layout.Add("B", "S", NULL);
  //cout << layout << endl;

  // static work
  DC_Work_t work;
  work.buf.sprintf("work = 100");
  work.buf.Append('\0');

  // determine where to place filters
  DC_Placement_t placement;
  placement.Add("A", "<one_per_node>");
  placement.Add("B", "<one_per_node>");

  int ret, wh;
  DC_FilterInstance_t *pinstance;
  cout << "starting instance ..." << endl;
  if ((ret = DC.NewFilterInstance(layout, work, placement, pinstance))
      != DC_ERR_OK) {
    cerr << "app: failed NewFilterInstance() rtn=" << ret << endl
	 << DC_strerror(ret) << endl;
    exit(1);
  }
  cout << "appending work ..." << endl;
  if ((wh = pinstance->AppendWork(work)) < 0) {
    cerr << "app: failed AppendWork() to instance rtn=" << wh << endl
	 << DC_strerror(wh) << endl;
  }
  cout << "waiting for work to finish ..." << endl;
  ret = DC.WaitWork(wh);
  cout << "wh=" << wh << " finished, ret=" << ret << endl;
  
  cout << "exiting ..." << endl;
  return 0;
}

//
// Test 3
//
// test to isolate problem with reader blocking on a full queue, and another
// filter is forced to block.  The queue size should be small (around 5) to see
// this happen.  The code at the top of thread-reader.cpp attempts to fix this
// by skipping reads for full queues.
int do_test3(int argc, char *argv[]) {
  DC_FilterService_t DC;
  if (DC.init("example-test", &filter_factory, &argc, &argv)) exit(1);

  // emulator args
  Global.wTotalSize = 4096;
  Global.wNumPiece = 20;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;

  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }
  // we are the console process

  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  // P1(fast) -> C1(slow)   disjoint   P2(slow) -> C2(fast)
  layout.Add("P1", NULL, "S1", false);
  layout.Add("C1", "S1", NULL);
  layout.Add("P2", NULL, "S2");
  layout.Add("C2", "S2", NULL, false);
  //cout << layout << endl;

  // static work
  DC_Work_t work;
  work.buf.sprintf("work = 100");
  work.buf.Append('\0');

  // determine where to place filters
  DC_Placement_t placement;
  placement.Add("P1", "<one_per_node>");
  placement.Add("P2", "<one_per_node>");
  placement.Add("C1", "<one_per_node>");
  placement.Add("C2", "<one_per_node>");
  //placement.Add("P1", "condor");
  //placement.Add("P2", "condor");
  //placement.Add("C1", "candida");
  //placement.Add("C2", "candida");

  int ret, wh;
  DC_FilterInstance_t *pinstance;
  cout << "starting instance ..." << endl;
  if ((ret = DC.NewFilterInstance(layout, work, placement, pinstance))
      != DC_ERR_OK) {
    cerr << "app: failed NewFilterInstance() rtn=" << ret << endl
	 << DC_strerror(ret) << endl;
    exit(1);
  }
  cout << "appending work ..." << endl;
  if ((wh = pinstance->AppendWork(work)) < 0) {
    cerr << "app: failed AppendWork() to instance rtn=" << wh << endl
	 << DC_strerror(wh) << endl;
  }
  cout << "waiting for work to finish ..." << endl;
  ret = DC.WaitWork(wh);
  cout << "wh=" << wh << " finished, ret=" << ret << endl;
  
  cout << "exiting ..." << endl;
  return 0;
}

//
// Test 4
//
// new test for user-defined transparent copy write choice
int do_test4(int argc, char *argv[]) {
  DC_FilterService_t DC;
  if (DC.init("example-test", &filter_factory, &argc, &argv)) exit(1);
  DC.setPolicyFactory(&policy_factory);

  // emulator args
  Global.wTotalSize = 4096;
  Global.wNumPiece = 20;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;

  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }
  // we are the console process

  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  layout.ReadFile("example-test_layout.cfg");
  //cout << layout << endl;

  // static work
  DC_Work_t work;
  work.buf.sprintf("work = 100");
  work.buf.Append('\0');

  // determine where to place filters
  DC_Placement_t placement;
  placement.ReadFile("example-test_placement.cfg");
  //cout << placement << endl;

  int ret, wh;
  DC_FilterInstance_t *pinstance;
  cout << "starting instance ..." << endl;
  if ((ret = DC.NewFilterInstance(layout, work, placement, pinstance))
      != DC_ERR_OK) {
    cerr << "app: failed NewFilterInstance() rtn=" << ret << endl
	 << DC_strerror(ret) << endl;
    exit(1);
  }
  cout << "appending work ..." << endl;
  if ((wh = pinstance->AppendWork(work)) < 0) {
    cerr << "app: failed AppendWork() to instance rtn=" << wh << endl
	 << DC_strerror(wh) << endl;
  }
  cout << "waiting for work to finish ..." << endl;
  ret = DC.WaitWork(wh);
  cout << "wh=" << wh << " finished, ret=" << ret << endl;
  
  cout << "exiting ..." << endl;
  return 0;
}

//
// Test 5
//
// new simple test for copyset subset support with write policies
int do_test5(int argc, char *argv[]) {
  DC_FilterService_t DC;
  if (DC.init("example-test5", &filter_factory, &argc, &argv)) exit(1);

  // emulator args
  Global.wTotalSize = 4096;
  Global.wNumPiece = 16;
  Global.rOverheadBuf  = 1;
  Global.rOverheadByte = 0;

  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }
  // we are the console process

  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  layout.ReadFile("example-test5_layout.cfg");
  //cout << "layout found:\n" << layout << endl;

  // static work
  DC_Work_t work;
  work.buf.sprintf("work = 100");
  work.buf.Append('\0');

  // determine where to place filters
  DC_Placement_t placement;
  placement.ReadFile("example-test5_placement.cfg");
  //cout << "placement found:\n" << placement << endl;

  int ret, wh;
  DC_FilterInstance_t *pinstance;
  cout << "starting instance ..." << endl;
  if ((ret = DC.NewFilterInstance(layout, work, placement, pinstance))
      != DC_ERR_OK) {
    cerr << "app: failed NewFilterInstance() rtn=" << ret << endl
	 << DC_strerror(ret) << endl;
    exit(1);
  }
  cout << "appending work ..." << endl;
  if ((wh = pinstance->AppendWork(work)) < 0) {
    cerr << "app: failed AppendWork() to instance rtn=" << wh << endl
	 << DC_strerror(wh) << endl;
  }
  cout << "waiting for work to finish ..." << endl;
  ret = DC.WaitWork(wh);
  cout << "wh=" << wh << " finished, ret=" << ret << endl;
  
  cout << "exiting ..." << endl;
  return 0;
}

int main(int argc, char *argv[]) {

#ifdef _WIN32
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
 
	wVersionRequested = MAKEWORD(2, 0);
 
	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) {
		cout << "Error, Winsock could not be initialized.";
		return 0;
	}  

#endif /* WIN32 */

  if (argc == 1) {
    cout << "please provide a test number from 1 to 5" << endl;
  } else {
    int i = atoi(argv[1]);
    switch (i) {
    case 1: do_test1(argc, argv); break;
    case 2: do_test2(argc, argv); break;
    case 3: do_test3(argc, argv); break;

    case 4: do_test4(argc, argv); break;
    case 5: do_test5(argc, argv); break;
    }
  }
  return 0;
}


// ****************************************************************************
