// ****************************************************************************
//  Name: example-thru.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 09/21/2000 : mdb - Created.
//
//  Desc: Sample filter application - Pipe-thru-console, stand alone.
// ****************************************************************************

#include "example-filters.h"

class P : public Producer {};
class F : public PassByFilter {};
class C : public Consumer {};

DC_Filter_Base_t *filter_factory(char *sbFilterName) {
  cout << "app: creating a new object for filter " << sbFilterName << endl;
  if (strcmp(sbFilterName, "P") == 0) {  
    return new P;
  } else if (strcmp(sbFilterName, "F") == 0) {  
    return new F;
  } else if (strcmp(sbFilterName, "C") == 0) {  
    return new C;
  } else {
    cerr << "app: unknown filter \"" << sbFilterName << "\" to create" << endl;
  }
  return NULL;
}


int main(int argc, char *argv[]) {

#ifdef _WIN32
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
 
	wVersionRequested = MAKEWORD(2, 0);
 
	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) {
		cout << "Error, Winsock could not be initialized.";
		return 0;
	}  

#endif /* WIN32 */

  DC_FilterService_t DC;
  if (DC.init("example-thru", &filter_factory, &argc, &argv)) exit(1);

  // emulator args
  Global.wTotalSize    = 4096;
  Global.wNumPiece     = 4;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;

  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }
  // we are the console process

  // describe set of filters and streams to execute
  DC_FilterLayout_t layout;
#if 0
  layout.ReadFile("example-thru_filterlayout.cfg");
#else
  layout.Add("P", NULL, "P-F");
  layout.Add("F", "P-F", "F-C");
  layout.Add("C", "F-C", "C-output");
  layout.Add("<console>", "C-output", NULL);
#endif

  // static work, and no client-server front end
  DC_Work_t work;
  work.buf.sprintf("query = 1 100 100 400 400 8");
  work.buf.Append('\0');
  cout << "main: work= " << work.buf << endl
       << ">" << work.buf.getPtr() << "<" << endl;

  // determine where to place filters
  DC_Placement_t placement;
  if (DC.PlacementPlanning(layout, work, placement) != 0) { 
    // we choose our own placement instead
    placement.Add("P", "<one_per_node>");
    placement.Add("F", "<one_per_node>");
    placement.Add("C", "<one_per_node>");
  }

  //
  // a simple static RPC style example
  //
  DC_FilterInstance_t *pinstance0 = NULL, *pinstance1 = NULL;
  DC_Buffer_t *pbuf;
  int ret, wh;
  
  // instantiate filters
  if ((ret = DC.NewFilterInstance(layout, work, placement, pinstance0))
      != DC_ERR_OK) {
    cerr << "app: failed GetFilterInstance() rtn=" << ret << endl
	 << DC_strerror(ret) << endl;
  } else if ((ret = DC.NewFilterInstance(layout, work, placement, pinstance1))
      != DC_ERR_OK) {
    cerr << "app: failed GetFilterInstance() rtn=" << ret << endl
	 << DC_strerror(ret) << endl;
  } else {
    int nAppend0 = 3, nAppend1 = 3;
    int i;
    // append work to the filter group instance 1
    for (i=nAppend0; i; --i) {
      work.buf.Empty();
      work.buf.sprintf("QUERY i0 work=%d hello XXX", nAppend0-i);
      work.buf.Append('\0');

      cout << "work=" << work.buf.getPtr() << endl;
      if ((wh = pinstance0->AppendWork(work, true)) < 0) {
	cerr << "app: failed AppendWork() to instance rtn=" << wh << endl
	     << DC_strerror(wh) << endl;
	--nAppend0;
      }
    }
    cout << "app: appended " << nAppend0 << " to filter group instance1"
	 << endl;
    // append work to the filter group instance 2
    for (i=nAppend1; i; --i) {
      work.buf.Empty();
      work.buf.sprintf("QUERY i1 work=%d XXX", nAppend0-i);
      work.buf.Append('\0');

      cout << "work=" << work.buf.getPtr() << endl;
      if ((wh = pinstance1->AppendWork(work, true)) < 0) {
	cerr << "app: failed AppendWork() to instance rtn=" << wh << endl
	     << DC_strerror(wh) << endl;
	--nAppend1;
      }
    }
    cout << "app: appended " << nAppend1 << " to filter group instance2"
	 << endl;

    //
    // 09/20/2001 : mdn - Fixed deadlock caused by console & instance.
    //
    // AppendWork() calls add work to an instance, and the result
    // can then be read by the <console> pseudo filter.  The problem
    // is if we don't know how many UOW to read, then we will
    // deadlock: the console will be stuck in a read, while the
    // remote filters will be awaiting more work.  We need to signal
    // there is no more work, but the previous version of the
    // StopFilterInstance() call was deleting the pinstance handle.
    // The detached case has been special cased to avoid making the
    // pinstance handle invalid for use in the
    // pinstance->arg.ins[#].read() calls to get the output.
    //
    // The end result correct usage is below for the known-num-uow
    // and the unknown-num-uow cases (STILL BROKEN).

#if 1
    // read exactly the number of expected sets of resulting data
    for (int nWork=nAppend0; nWork>0; --nWork) {
      cout << "app: i0 reading work " << nAppend0-nWork+1
	   << " of " << nAppend0 << endl;
      while (1) {
	if ((pbuf = pinstance0->arg.ins[0].read()) == NULL) {
	  if (pinstance0->arg.ins[0].isEndOfStream()) {
	    cout << pinstance0->arg.sbFullName << ": EOS seen" << endl;
	  } else {
	    cout << pinstance0->arg.sbFullName << ": EOW boundary" << endl;
	  }
	  break;
	}
#if defined(DCDBG_READWRITE)
#if defined(USE_STRING_BUFS)
	cout << pinstance0->arg.sbFullName << "::read " << pbuf->getPtr()
	     << endl;
#else
	cout << pinstance0->arg.sbFullName << "::read " << *pbuf << endl;
#endif
#endif /* DCDBG_READWRITE */
	// TODO: do something with the result
	pbuf->consume();
      }
    }
    cout << "app: stopping i0" << endl;
    // since we knew exactly how many were written, we can wait
    // until now to stop the instance
    DC.StopFilterInstance(pinstance0);

    // read exactly the number of expected sets of resulting data
    for (int nWork=nAppend1; nWork>0; --nWork) {
      cout << "app: i1 reading work " << nAppend1-nWork+1
	   << " of " << nAppend1 << endl;
      while (1) {
	if ((pbuf = pinstance1->arg.ins[0].read()) == NULL) {
	  if (pinstance1->arg.ins[0].isEndOfStream()) {
	    cout << pinstance1->arg.sbFullName << ": EOS seen" << endl;
	  } else {
	    cout << pinstance1->arg.sbFullName << ": EOW boundary" << endl;
	  }
	  break;
	}
#if defined(DCDBG_READWRITE)
#if defined(USE_STRING_BUFS)
	cout << pinstance1->arg.sbFullName << "::read " << pbuf->getPtr()
	     << endl;
#else
	cout << pinstance1->arg.sbFullName << "::read " << *pbuf << endl;
#endif
#endif /* DCDBG_READWRITE */
	// TODO: do something with the result
	pbuf->consume();
      }
    }
    cout << "app: stopping i1" << endl;
    // since we knew exactly how many were written, we can wait
    // until now to stop the instance
    DC.StopFilterInstance(pinstance1);
#else
    // after appended work has been read, stop the instance
    // (note: when using the DD policy, we *must* call this with
    //  fWait==false if we have not read all the data from the
    //  stream.  If we do not, deadlock may result)
    DC.StopFilterInstance(pinstance0, false);

    // keep reading the resulting data until EOS
    while (1) {
      if ((pbuf = pinstance0->arg.ins[0].read()) == NULL) {
	if (pinstance0->arg.ins[0].isEndOfStream()) {
	  cout << pinstance0->arg.sbFullName << ": EOS seen" << endl;
	  break;
	}
	cout << pinstance0->arg.sbFullName << ": EOW boundary" << endl;
	continue;
      }
#if defined(DCDBG_READWRITE)
      cout << pinstance0->arg.sbFullName << "::read " << *pbuf << endl;
#endif /* DCDBG_READWRITE */
      // TODO: do something with the result
      pbuf->consume();
    }
#endif
  }
  return 0;
}


// ****************************************************************************
