// ****************************************************************************
//  Name: detach_filters.cpp
//  Athr: Mike Gray (mgray)
//  Date: 05/21/2002 : mgray - Created.
//
//  Desc: Sample dynamic library for filters
// ****************************************************************************

#include <DataCutter.h>
#include <lib/dc_timer.h>
#include "example-filters.h"

/* 
 * _init and _fini are two functions that are automatically called
 * when your shared library is loaded and unloaded.  this is an 
 * appropriate place to put any startup/shutdown code you may need.
 */
extern "C"
void _init()
{
  /* called during dlopen() */
  cout << "detach_filters _init called" << endl;

  // emulator args
  Global.wTotalSize    = 4096;
  Global.wNumPiece     = 4;
  //Global.wTotalSize    = 4096*1024*5;
  //Global.wNumPiece     = 32;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;

}

extern "C"
void _fini()
{
  /* called during dlclose() */
  cout << "detach_filters _fini called" << endl;
}


/*
 * this is the shared library entry point that DataCutter uses to 
 * try to instantiate filters by name.  if you implement a filter
 * with the name given in the parameter, you should return an 
 * instance of the class, otherwise you should return NULL.
 */
extern "C"
DC_Filter_Base_t *filter_factory(char *sbFilterName) {
  cout << "detach_filters: creating a new object for filter " << sbFilterName << endl;
  if (strcmp(sbFilterName, "P") == 0 ||
      strcmp(sbFilterName, "Producer") == 0) {
    return new Producer;
  } else if (strcmp(sbFilterName, "F") == 0 ||
	     strcmp(sbFilterName, "PassByFilter") == 0) {
    return new PassByFilter;
  } else if (strcmp(sbFilterName, "C") == 0 ||
	     strcmp(sbFilterName, "Consumer") == 0) {
    return new Consumer;
  } else if (strcmp(sbFilterName, "NullFilter") == 0) {
    return new NullFilter;
  } else {
    cerr << "app: unknown filter \"" << sbFilterName << "\" to create" << endl;
  }
  return NULL;
}

