#include <assert.h>
 
#include "../src/DataCutter.h"
#include "../lib/dc_timer.h"
#include "../lib/dc_random.h"
#include "dc_ctojava_filter_t.h"

struct Global_s {
  int wTotalSize;
  int wNumPiece;
  double rOverheadBuf;
  double rOverheadByte;
} Global;



//class DC_Filter_Test : public DC_Filter_Base_t {
//
//public:
//    int init(DC_Filter_Base_t::initarg_t& arg) {return 0;};
//
//    DC_RTN_t process(DC_Filter_Base_t::arg_t& arg) {
//        return 1;
//    };
//
//    int finalize() { return 0;};
//};

DC_Filter_Base_t *filter_factory(char *sbFilterName) { 
  Timer t;
  DC_Filter_Base_t *x;
  
  cout << "app: creating a new object for filter " << sbFilterName << endl; 

  //Here should try to create C++ filters
  t.Reset();
  t.Start();
  cout << "Timer started" << endl;

  //Check to see if the filter exists in Java
  x = DC_CToJava_Filter_t::filter_factory(sbFilterName);
  cout << "Time taken to createa a Java filter: " << t.Elapsed() << endl;
  return x;
}
 
int main(int argc, char *argv[]) {
  //Timer t;

  //t.Reset();
  //t.Start();
  //NativeReg::ConstructNReg();
  //cout << "JVM Creation took " << t.Elapsed() << endl;

  DC_FilterService_t DC; 
  if (DC.init("example-JNI-one", &filter_factory, &argc, &argv)) exit(1);
   
  // emulator args 
  Global.wTotalSize    = 4096;
  Global.wNumPiece     = 4;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;
   
  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess(); 
    return 0; 
  } 
  // we are the console process
   
  // describe set of filters and streams to execute 
  DC_FilterLayout_t layout;
  layout.Add("DC_Filter_Test", NULL, NULL);

  cout << layout << endl; 
   
  // static work 
  DC_Work_t work;
  work.buf.sprintf("work = 100"); 
  work.buf.Append('\0'); 
     
  // determine where to place filters
  DC_Placement_t placement;
  placement.Add("DC_Filter_Test", "<one_per_node>");

  int ret, wh;
  DC_FilterInstance_t *pinstance;
  cout << "starting instance ..." << endl;
  if ((ret = DC.NewFilterInstance(layout, work, placement,
pinstance))
      != DC_ERR_OK) {
    cerr << "app: failed NewFilterInstance() rtn=" << ret << endl
         << DC_strerror(ret) << endl;
    exit(1);
  }
  cout << "appending work ..." << endl;
  if ((wh = pinstance->AppendWork(work)) < 0) {
    cerr << "app: failed AppendWork() to instance rtn=" << wh <<
endl
         << DC_strerror(wh) << endl;
  }
  cout << "waiting for work to finish ..." << endl;
  ret = DC.WaitWork(wh);
  cout << "wh=" << wh << " finished, ret=" << ret << endl;

  cout << "exiting ..." << endl;

  return 0;
}   
