#include <assert.h>
 
#include "../src/DataCutter.h"
#include "../lib/dc_timer.h"
#include "../lib/dc_random.h"
#include "dc_ctojava_filter_t.h"

struct Global_s {
  int wTotalSize;
  int wNumPiece;
  double rOverheadBuf;
  double rOverheadByte;
} Global;


class DC_Filter_SendOutput : public DC_Filter_Base_t {
  
public:
  DC_RTN_t init(DC_Filter_Base_t::initarg_t& arg) {return DC_RTN_OK;};
  
  DC_RTN_t process(DC_Filter_Base_t::arg_t& arg) {
    DC_Buffer_t* x[5];
    x[0] = new DC_Buffer_t(10);
    for (int i = 0; i < 10; i++)  {
      x[0]->Append("A", 1);
    }
    
    x[1] = new DC_Buffer_t(100);
    for (int i = 0; i < 100; i++)  {
      x[1]->Append("B",1 );
    }
    
    x[2] = new DC_Buffer_t(1000);
    for (int i = 0; i < 1000; i++)  {
      x[2]->Append("C",1 );
    }
    
    x[3] = new DC_Buffer_t(10000);
    for (int i = 0; i < 10000; i++)  {
      x[3]->Append("D", 1);
    }
    
    x[4] = new DC_Buffer_t(100000);
    for (int i = 0; i < 100000; i++) {
      x[4]->Append("E", 1);
    }
    
    for (int i = 0; i < 5; i++) {
      Timer t;
      
      t.Start();
      arg.outs[0].write_nocopy((*x[i]));
      cout << "Time taken to write in C++ " << t.Elapsed() << endl;
    }
    arg.outs[0].close();
    
    return DC_RTN_OK;
    //return 1;
  };
  
  DC_RTN_t finalize() { cout << "FINISHED C++ filter" << endl; return DC_RTN_OK;};
};

class DC_Filter_GetInput : public DC_Filter_Base_t {
  
public:
  DC_RTN_t init(DC_Filter_Base_t::initarg_t& arg) {return DC_RTN_OK;};
  
  DC_RTN_t process(DC_Filter_Base_t::arg_t& arg) {
    
    cout << "Starting wait" << endl;
    CpuDelay delay(30);
    delay.BusyWait(30);
    cout << "Resuming" << endl;
    
    for (int i = 0; i < 5; i++) {	
      Timer t;
      
      t.Start();
      DC_Buffer_t *x = arg.ins[0].read();
      cout << "Time taken to read buffer of size " << x->getSize() << " in C++ is " << t.Elapsed() << endl;
    }
    
    arg.ins[0].close();
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize() { return DC_RTN_OK;};
};

DC_Filter_Base_t *filter_factory(char *sbFilterName) { 
  Timer t;
  DC_Filter_Base_t *x;
  cout << "app: creating a new object for filter " << sbFilterName << endl; 
  
  // Here should try to create C++ filters
  //  t.Reset();
  //  t.Start(); 
  //  if (strcmp(sbFilterName, "DC_Filter_GetInput") == 0) {
  //     x = new DC_Filter_GetInput();
  //     cout << "Created C++ Filter in: " << t.Elapsed() << endl;
  //     return x;
  //  }
  
  //  t.Reset();
  //  t.Start();
  //  if (strcmp(sbFilterName, "DC_Filter_SendOutput") == 0) {
  //     x = new DC_Filter_SendOutput();
  //     cout << "Created C++ Filter in: " << t.Elapsed() << endl;
  //     return x;
  //  }
  
  t.Reset();
  t.Start();
  //Check to see if the filter exists in Java
  x = DC_CToJava_Filter_t::filter_factory(sbFilterName); 
  cout << "Created Java Filter in " << t.Elapsed() << endl;
  return x;
}

int main(int argc, char *argv[]) {
  //Timer t;
  
  //t.Reset();
  //t.Start();
  //NativeReg::ConstructNReg();
  //cout << "JVM Loading took " << t.Elapsed() << endl;
  
  DC_FilterService_t DC; 
  if (DC.init("example-JNI", &filter_factory, &argc, &argv)) exit(1);
  
  // emulator args 
  Global.wTotalSize    = 4096;
  Global.wNumPiece     = 4;
  Global.rOverheadBuf  = 0;
  Global.rOverheadByte = 0;
  
  // check if this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // run any filter instances as directed by console process
    DC.RemoteProcess(); 
    return 0; 
  } 
  // we are the console process
  
  // describe set of filters and streams to execute 
  DC_FilterLayout_t layout;
  layout.Add("DC_Filter_SendOutput", NULL, "DC_Filter_SendOutput-DC_Filter_GetInput");
  layout.Add("DC_Filter_GetInput", "DC_Filter_SendOutput-DC_Filter_GetInput", NULL);
  
  cout << layout << endl; 
  
  // static work 
  DC_Work_t work;
  work.buf.sprintf("work = 100"); 
  work.buf.Append('\0'); 
  
  // determine where to place filters
  DC_Placement_t placement;
  placement.Add("DC_Filter_SendOutput", "<one_per_node>", 1);
  placement.Add("DC_Filter_GetInput", "<one_per_node>", 1);
  
  
  // get a filter instance (existing or new) for executing work
  DC_FilterInstance_t *pinstance = NULL;
  int ret, wh;
  for (int repeat=0; repeat<1; ++repeat) {
    if ((ret = DC.GetFilterInstance(layout, work, placement, pinstance))
        != DC_ERR_OK) {
      cerr << "app: failed GetFilterInstance() rtn=" << ret << endl
           << DC_strerror(ret) << endl;
    } else {
      // append the work
      cout << "Appended work to filter" << endl;
      if ((wh = pinstance->AppendWork(work)) < 0) {
        cerr << "app: failed AppendWork() to instance rtn=" << wh << endl
             << DC_strerror(wh) << endl;
      }
    }
  }
  
  
  // wait for all work to finish in any order
  while ((wh = DC.WaitAnyWork()) != DC_ERR_NoWorkFound) {
    if (wh < 0) {
      cerr << "app: failed WaitAnyWork() call" << DC_strerror(wh) <<
	endl;
    } else {
      cout << "app: wh=" << wh << " finished, ret="
	   << DC.ExitStatus(wh) << endl;
    }
  }
  
  return 0;
}   
