//
// test-buffer.cpp
//

#include "test-filters.h"

int main(int argc, char* argv[]) {
  DC_FilterService_t DC;
  if (DC.init("test-buffer", &filter_factory, &argc, &argv)) {
    exit(1);
  }

  // Check whether this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // Run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }

  // This is the console process

  // Describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  layout.Add("Muse", NULL, "Muse-Poet");
  layout.Add("Poet", "Muse-Poet", NULL);

  // Describe where to place filters
  DC_Placement_t placement;
  placement.Add("Muse", DIRD_DEFAULT_HOST);
  placement.Add("Poet", DIRD_DEFAULT_HOST);

  // Static work, and no client-server front end
  DC_Work_t work;
  work.buf.Empty();
  
  // Get a filter instance (existing or new) for executing work
  int status;
  DC_FilterInstance_t* instance;
  if ((status = DC.NewFilterInstance(layout, work, placement, instance)) != DC_ERR_OK) {
    cerr << "failed GetFilterInstance: " << DC_strerror(status) << endl;
    return 1;
  }
  
  // Append work to instance and await its completion
  int wh;
  if ((wh = instance->AppendWork(work)) < 0) {
    cerr << "failed AppendWork: " << DC_strerror(wh) << endl;
    return 1;
  }
  DC.WaitWork(wh);
  return 0;
}

