#include <DataCutter.h>
#include "dird.h"

#ifndef _WIN32
	#include <sys/time.h>
#endif

#ifdef _WIN32
	#include <win32.h>
	#include <unix_calls.H>
#endif

const int LINES = 34;

char* carroll[LINES] = {
  "Twas brillig, and the slithy toves",
  "Did gyre and gimble in the wabe:",
  "All mimsy were the borogoves,",
  "And the mome raths outgrabe.",
  "",                                     
  "\"Beware the Jabberwock, my son!",
  "The jaws that bite, the claws that catch!",
  "Beware the Jubjub bird, and shun",
  "The frumious Bandersnatch!\"",
  "",                                       
  "He took his vorpal sword in hand:",
  "Long time the manxome foe he sought--",
  "So rested he by the Tumtum tree,",
  "And stood awhile in thought.",
  "",                                    
  "And, as in uffish thought he stood,",
  "The Jabberwock, with eyes of flame,",
  "Came whiffling through the tulgey wood,",
  "And burbled as it came!",
  "",                                       
  "One, two! One, two! And through and through",
  "The vorpal blade went snicker-snack!",
  "He left it dead, and with its head",
  "He went galumphing back.",
  "",
  "\"And hast thou slain the Jabberwock?",
  "Come to my arm, my beamish boy!",
  "O frabjous day! Callooh! Callay!\"",
  "He chortled in his joy.",
  "",
  "'Twas brillig, and the slithy toves",
  "Did gyre and gimble in the wabe:",
  "All mimsy were the borogoves",
  "And the mome raths outgrabe."
};

// Does absolutely nothing
class Null: public DC_Filter_Base_t {
 public:
  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  }
  
  DC_RTN_t process(arg_t &arg) {
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

// Echos the work definition
class Echo: public DC_Filter_Base_t {
 public:
  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  };
  
  DC_RTN_t process(arg_t &arg) {
    char* msg = arg.pwork->buf.getPtr();
    
    cout << "Work definition: " << msg << endl;
    return DC_RTN_OK;
  }

  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

const int BUFSIZE = 1024;

// Divines a line of poetry
class Muse : public DC_Filter_Base_t {
public:
  int cnt;
  DC_Buffer_t buf;

  Muse(void):cnt(1) {
    cout << endl;
  }

  DC_RTN_t init(initarg_t &initarg) {
    buf.New(BUFSIZE);
    return DC_RTN_OK;
  }

  DC_RTN_t process(arg_t &arg) {
    timeval tv;
    gettimeofday(&tv, NULL);
    srand((unsigned)tv.tv_usec);
    int i = rand() % LINES;
    buf.Append(carroll[i], (Types::uint)strlen(carroll[i]));
    buf.Append("\0", 1);
    buf.setSize((Types::uint)strlen(carroll[i]) + 1);
    if (arg.outs[0].write(&buf) != DC_ERR_OK) {
      cerr << arg.sbFilterName << ": failed write to buffer" << endl;
    }
    cout << "<muse:" << cnt << "> " << buf.getPtr() << endl;
    ++cnt;
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

// Divines multiple lines of poetry
class ProlificMuse : public DC_Filter_Base_t {
public:
  int cnt;
  DC_Buffer_t buf;
  
  ProlificMuse(void):cnt(1) {
    cout << endl;
  }

  DC_RTN_t init(initarg_t &initarg) {
    buf.New(BUFSIZE);
    return DC_RTN_OK;
  }

  DC_RTN_t process(arg_t &arg) {
    timeval tv;
    int len;

    arg.pwork->buf.Extract(&len);
    gettimeofday(&tv, NULL);
    srand((unsigned)tv.tv_usec);
    for (int i = 0; i < len; ++i) {
      int line = rand() % LINES;
      buf.Empty();
      buf.Append(carroll[line], (Types::uint)strlen(carroll[line]));
      buf.Append("\0", 1);
      buf.setSize((Types::uint)strlen(carroll[line]) + 1);
      if (arg.outs[0].write(&buf) != DC_ERR_OK) {
	cerr << arg.sbFilterName << ": failed write to buffer" << endl;
      }
      cout << "<muse:" << cnt << "> " << buf.getPtr() << endl;
    }
    ++cnt;
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

// Divines a line of poetry
class FaultyMuse : public DC_Filter_Base_t {
 public:
  int cnt;
  DC_Buffer_t buf;
  
  FaultyMuse(void):cnt(1) {
    cout << endl;
  }

  DC_RTN_t init(initarg_t &initarg) {
    buf.New(BUFSIZE);
    return DC_RTN_OK;
  }
  
  DC_RTN_t process(arg_t &arg) {
    timeval tv;
    gettimeofday(&tv, NULL);
    srand((unsigned)tv.tv_usec);
    int i = rand() % LINES;

    if (*carroll[i] == 'A') {
      return DC_RTN_CONT;
    } else if (*carroll[i] == '\0') {
      return DC_RTN_ABRT;
    }
    buf.Append(carroll[i], (Types::uint)strlen(carroll[i]));
    buf.Append("\0", 1);
    buf.setSize((Types::uint)strlen(carroll[i]) + 1);
    if (arg.outs[0].write(&buf) != DC_ERR_OK) {
      cerr << arg.sbFilterName << ": failed write to buffer" << endl;
    }
    cout << "<muse:" << cnt << "> " << buf.getPtr() << endl;
    ++cnt;
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

// Speaks a line of poetry
class Poet : public DC_Filter_Base_t {
 public:
  int cnt;

  Poet(void):cnt(1) {
    cout << endl;
  }
  
  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  };
  
  DC_RTN_t process(arg_t &arg) {
    DC_Buffer_t *pbuf;

    while ((pbuf = arg.ins[0].read())) {
      cout << "<poet:" << cnt << "> " << pbuf->getPtr() << endl;
      pbuf->consume();
    }
    ++cnt;    
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

// Speaks a line of poetry
class FaultyPoet : public DC_Filter_Base_t {  
 public:
  int cnt;

  FaultyPoet(void):cnt(1) {
    cout << endl;
  }

  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  };
  
  DC_RTN_t process(arg_t &arg) {
    DC_Buffer_t *pbuf;

    while ((pbuf = arg.ins[0].read())) {
      if (*(pbuf->getPtr()) == 'A') {
	return DC_RTN_CONT;
      }
      if (*(pbuf->getPtr()) == '\0') {
	return DC_RTN_ABRT;
      }
      cout << "<poet:" << cnt << "> " << pbuf->getPtr() << endl;
      pbuf->consume();
      ++cnt;
    }    
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

class Merge : public DC_Filter_Base_t {
 public:
  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  };
  
  DC_RTN_t process(arg_t &arg) {
    DC_Buffer_t *pbuf;
    
    while ((pbuf = arg.insReadAny())) {
      cout << "<merge> " << pbuf->getPtr() << endl;
      arg.outs[0].write(pbuf);
      pbuf->consume();
    }
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

class Split : public DC_Filter_Base_t {
 public:
  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  };
  
  DC_RTN_t process(arg_t &arg) {
    int i;
    DC_Buffer_t *pbuf;
    
    while ((pbuf = arg.ins[0].read())) {
      for (i = 0; i < arg.nouts; ++i) {
	cout << "<split> " << pbuf->getPtr() << endl;
	if (arg.outs[i].write(pbuf) != DC_ERR_OK) {
	  cout << "error on write" << endl;
	  arg.outs[i].close();
	  //return DC_RTN_OK;
	}
      }
      pbuf->consume();
    }
    return DC_RTN_OK;
  };
  
  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

class Delay: public DC_Filter_Base_t {
 public:
  int cnt;

  Delay(void):cnt(1) {
    cout << endl;
  }
  
  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  };
  
  DC_RTN_t process(arg_t &arg) {
    DC_Buffer_t *pbuf;
    int len;

    arg.pwork->buf.Extract(&len);
    while ((pbuf = arg.ins[0].read())) {
      if (*(pbuf->getPtr()) == '\0') {
	++cnt;
	return DC_RTN_CONT;
      }
      cout << "<delay:" << cnt << "> iteration " << len << endl;
      //sleep(len * 10);
      sleep(5);
      arg.outs[0].write(pbuf);
      pbuf->consume();
    }
    ++cnt;
    return DC_RTN_OK;
  }

  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

class NoDelay: public DC_Filter_Base_t {
 public:
  int cnt;

  NoDelay(void):cnt(1) {
    cout << endl;
  }

  DC_RTN_t init(initarg_t &initarg) {
    return DC_RTN_OK;
  };
  
  DC_RTN_t process(arg_t &arg) {
    DC_Buffer_t *pbuf;
    int len;

    arg.pwork->buf.Extract(&len);
    while ((pbuf = arg.ins[0].read())) {
      if (*(pbuf->getPtr()) == '\0') {
	++cnt;
	return DC_RTN_CONT;
      }
      cout << "<delay:" << cnt << "> iteration " << len << endl;
      //sleep(len * 10);
      arg.outs[0].write(pbuf);
      pbuf->consume();
    }
    ++cnt;
    return DC_RTN_OK;
  }

  DC_RTN_t finalize(void) {
    return DC_RTN_OK;
  };
};

// Filter instantiation routine
DC_Filter_Base_t *filter_factory(char *sbFilterName) {
  if (strcmp(sbFilterName, "Null") == 0) {
    return new Null;
  } else if (strcmp(sbFilterName, "Echo") == 0) {
    return new Echo;
  } else if (strcmp(sbFilterName, "Muse") == 0) {
    return new Muse;
  } else if (strcmp(sbFilterName, "FaultyMuse") == 0) {
    return new FaultyMuse;
  } else if (strcmp(sbFilterName, "ProlificMuse") == 0) {
    return new ProlificMuse;
  } else if (strcmp(sbFilterName, "Poet") == 0) {
    return new Poet;
  } else if (strcmp(sbFilterName, "FaultyPoet") == 0) {
    return new FaultyPoet;
  } else if (strcmp(sbFilterName, "Merge") == 0) {
    return new Merge;
  } else if (strcmp(sbFilterName, "Split") == 0) {
    return new Split;
  } else if (strcmp(sbFilterName, "Delay") == 0) {
    return new Delay;
  } else if (strcmp(sbFilterName, "NoDelay") == 0) {
    return new NoDelay;
  } else {
    cerr << "unknown filter: " << sbFilterName << endl;
  }
  return NULL;
}
