//
// test-instance.cpp
//

#include "test-filters.h"

const int INSTANCES = 10;
const int JOBS = 20;
const int SETS = 10;

int main(int argc, char* argv[]) {
  int sts;
  DC_FilterService_t DC;

  if (DC.init("test-instance", &filter_factory, &argc, &argv)) {
    exit(1);
  }

  // Check whether this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // Run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }

  // This is the console process

  // Describe set of filters and streams to execute
  DC_FilterLayout_t layout;
  layout.Add("Echo", NULL, NULL);

  // Describe where to place filters
  DC_Placement_t placement;
  placement.Add("Echo", DIRD_DEFAULT_HOST);

  // Static work, and no client-server front end
  DC_Work_t work;
  work.buf.Empty();
  work.buf.Append("test-instance\0", 14);

  // Get a filter instance (existing or new) for executing work
  DC_FilterInstance_t* instance[INSTANCES];

  for (int i = 0; i < INSTANCES; ++i) {
    cout << "starting instance " << i << " ..." << endl;
    sts = DC.NewFilterInstance(layout, work, placement, instance[i]);
    if (sts != DC_ERR_OK) {
      cerr << "failed GetFilterInstance: " << DC_strerror(sts) << endl;
      return 1;
    }
  }
  
  int wh[INSTANCES];
  for (int i = 0; i < INSTANCES; ++i) {
    // Append work to instance and await its completion
    if ((wh[i] = instance[i]->AppendWork(work)) < 0) {
      cerr << "failed AppendWork: " << DC_strerror(wh[i]) << endl;
      return 1;
    }
  }

  for (int i = 0; i < INSTANCES; ++i) {
    DC.WaitWork(wh[i]);
    sts = DC.StopFilterInstance(instance[i], true);
    if (sts != DC_ERR_OK) {
      cerr << "failed StopFilterInstance: " << DC_strerror(sts) << endl;
      return 1;
    }
  }
  return 0;
}
