//
// test-returns.cpp
//

#include "test-filters.h"

int main(int argc, char* argv[]) {
  int queries = 1, rounds = 1, setup = 1;
  DC_FilterService_t DC;

  if (argc > 3) {
    rounds = atoi(argv[3]);
  }
  if (argc > 2) {
    queries = atoi(argv[2]);
  }
  if (argc > 1) {
    setup = atoi(argv[1]);
  }
  if (DC.init("test-returns", &filter_factory, &argc, &argv)) {
    exit(1);
  }

  // Check whether this process is a remote copy for running filter instances
  if (DC.isRemoteProcess()) {
    // Run any filter instances as directed by console process
    DC.RemoteProcess();
    return 0;
  }

  // This is the console process
  DC_FilterLayout_t layout;
  DC_Placement_t placement;

  switch(setup) {
  case 1:
    // Describe set of filters and streams to execute
    layout.Add("Muse", NULL, "MtoS");
    layout.Add("Split", "MtoS", "StoP1 StoP2");
    layout.Add("Poet", "StoP1", NULL);
    layout.Add("FaultyPoet", "StoP2", NULL);
    
    // Describe where to place filters
    placement.ReadFile("test-split.cfg");
    break;
  case 2:
    // Describe set of filters and streams to execute
    layout.Add("Muse", NULL, "M1toM");
    layout.Add("FaultyMuse", NULL, "M2toM");
    layout.Add("Merge", "M1toM M2toM", "MtoP");
    layout.Add("Poet", "MtoP", NULL);
    
    // Describe where to place filters
    placement.ReadFile("test-merge.cfg");
    break;
  case 3:
    // Describe set of filters and streams to execute
    layout.Add("ProlificMuse", NULL, "MtoD");
    layout.Add("NoDelay", "MtoD", "DtoD");
    layout.Add("Delay", "DtoD", "DtoP");
    layout.Add("Poet", "DtoP", NULL);
    
    // Describe where to place filters
    placement.ReadFile("test-delay.cfg");
    break;
  default:
    cout << "whoops, unknown setup" << endl;
    return 1;
  }

  // Static work, and no client-server front end
  DC_Work_t work;
  work.buf.Empty();

  // Get a filter instance (existing or new) for executing work
  int i, j, status;
  DC_FilterInstance_t* instance;
  
  for (i = 0; i < rounds; ++i) {
    if ((status = DC.NewFilterInstance(layout, work, placement, instance)) != DC_ERR_OK) {
      cerr << "failed NewFilterInstance: " << DC_strerror(status) << endl;
      return 1;
    }
    
    // Append work to instance and await its completion
    int wh;
    for (j = 1; j <= queries; ++j) {
      work.buf.Empty();
      work.buf.Append(j);
      if ((wh = instance->AppendWork(work)) < 0) {
	cerr << "failed AppendWork: " << DC_strerror(wh) << endl;
	return 1;
      }
      
      status = DC.WaitWork(wh);
      cerr << "work status: " << DC_strerror(status) << endl;
      if (status == DC_ERR_RemoteFilterFail || status == DC_ERR_InstanceFail) {
	break;
      }
    }
    if ((status = DC.StopFilterInstance(instance)) != DC_ERR_OK) {
      cerr << "failed StopFilterInstance: " << DC_strerror(status) << endl;
    }
  }
  return 0;
}
