// ****************************************************************************
//  Name: cfg_util.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 01/07/2000 : mdb - Created.
//        04/14/2000 : mdb - Added -merge.
//        01/04/2002 : mdb - Added extract to split filterstats output into
//                           seperate instance stat sets.
//  Desc: Standlaone utility for handling cfg files (since they are getting
//        more complicated).
// ****************************************************************************

#include "lib/dc_standard.h"
#include "lib/dc_configinfo.h"

#if defined(DCDBG)
//#define DCDBG_CFG_READ
#endif /* DCDBG */


void usage(char *sb = NULL, int wExit = -1) {
  if (sb) {
    cerr << ErrorHeader(wERROR) << sb << endl;
  }
  cerr << "usage: "
       << sbProgramName << " <command>" << endl
       << "   commands: dump <configfile>" << endl
       << "             find <configfile> <section> <name>" << endl
       << "             merge <configfile1> <configfile2> ... " << endl
       << "             extract <configfile> <name> [<value>]" << endl
       << "             atoi <string>" << endl
       << endl;
  if (wExit != -1) exit(wExit);
}


int main(int argc, char *argv[]) {
  ConfigInfo cfg;
  char *sb;

  (void) StandardInit(argc, argv);

  if (argc < 3) usage(NULL, 1);

  if (strcmp(argv[1], "dump") == 0) {
    if (argc != 3) usage("wrong number of arguments for dump", 1);
    if (cfg.ReadFile(argv[2]) != 0) return 1;
    cout << cfg << endl;

  } else if (strcmp(argv[1], "find") == 0) {
    if (argc != 5) usage("wrong number of arguments for find", 1);
    if (cfg.ReadFile(argv[2]) != 0) return 1;
    sb = cfg.Find(argv[3], argv[4]);
    if (sb) cout << sb << endl;

  } else if (strcmp(argv[1], "merge") == 0) {
    if (argc < 4) usage("wrong number of arguments for merge", 1);
    if (cfg.ReadFile(argv[2]) != 0) return 1;
    for (int i=3; i<argc; ++i) {
      ConfigInfo cfgTmp;
      cfgTmp.ReadFile(argv[i]);
      // For now, we use the default merge options.  If we ever need
      // diff settings the easiest would be to add a merge2 command.
      cfg.Merge(cfgTmp);
    }
    cout << cfg << endl;

  } else if (strcmp(argv[1], "extract") == 0) {
    if (argc != 4 && argc != 5)
      usage("wrong number of arguments for extract", 1);
    cfg.SetOpts(ConfigInfo::ADD_DUPSEC_APPEND |
		ConfigInfo_Section::ADD_DUPNAME_APPEND);
    if (cfg.ReadFile(argv[2]) != 0) return 1;
    const char *sbSection, *sbName;
    bool fExtract;
    bool fTestValue = (argc == 5 ? true : false);
    bool fSpace = false;
    sbSection = cfg.FirstSection();
    while (sbSection != NULL) {
      fExtract = false;
      // check if the required name[=value] is in this section
      sbName = cfg.FirstName();
      while (sbName != NULL) {
	if (strcmp(sbName, argv[3]) == 0 &&
	    (!fTestValue || strcmp(cfg.CurrentValue(), argv[4]) == 0)) {
	  fExtract = true;
	  break;
	}
	sbName = cfg.NextName();
      }
      if (fExtract) {
	// output this section
	if (fSpace) {
	  cout << endl;
	} else {
	  fSpace = true;
	}
	cout << '[' << sbSection << ']' << endl;
	sbName = cfg.FirstName();
	while (sbName != NULL) {
	  cout << sbName << "=" << cfg.CurrentValue() << endl;
	  sbName = cfg.NextName();
	}
      }
      sbSection = cfg.NextSection();
    }

  } else if (strcmp(argv[1], "atoi") == 0) {
    if (argc != 3) usage("wrong number of arguments for atoi", 1);
    cout << Atoi(argv[2]) << endl;

  } else {
    usage("unknown command");
    exit(1);
  }

  (void) StandardCleanup();
  cout.flush();
  cerr.flush();
  return 0;
}


// ****************************************************************************
