// -*- c++ -*-
// ****************************************************************************
//  Name: dc_digraph.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 06/01/2001 : mdb - Created.
//  Desc: Simple directed graph class.
// ****************************************************************************

#if !defined(H_DIGRAPH)
#define H_DIGRAPH

#include "lib/dc_standard.h"
#include "lib/dc_darray.h"


// ****************************************************************************
//  Class: DC_Graph_t
// ****************************************************************************

class DC_Edge_Base {
public:
  Types::uint i;  // our index in enclosing da
  Types::uint iu;  // daVertex index of src u for edge (u,v)
  Types::uint iv;  // daVertex index of dst v for edge (u,v)
  DC_Edge_Base *pNext; // links of adjacency list

  // standard properties
  char  *sbName;
  bool   fHidden;
  double rWeight;

public:
  DC_Edge_Base(void);
  ~DC_Edge_Base(void);
};

class DC_Vertex_Base {
  friend ostream &operator<<(ostream &os, DC_Vertex_Base &v);
public:
  Types::uint i;  // our index in enclosing da
  DC_Edge_Base *pHead; // links of adjacency list

  // standard properties
  char  *sbName;
  bool   fHidden;
  double rWeight;

public:
  DC_Vertex_Base(void);
  ~DC_Vertex_Base(void);
};

// basic graph data structure - adjacency list
template <class VertexClass = DC_Vertex_Base,
	  class EdgeClass   = DC_Edge_Base>
class DC_Graph_AdjacencyList_t {
  //  friend ostream &operator<<(ostream &os, DC_Graph_AdjacencyList_t &G);
protected:
  DArrayPtr<DC_Vertex_Base *> daVertex;
  DArrayPtr<DC_Edge_Base *>   daEdge;
public:
  DC_Graph_AdjacencyList_t(void) {};
  ~DC_Graph_AdjacencyList_t(void) {};

  // rtn: DC_Edge_Base::ie of edge newly inserted or previously existing
  Types::uint add_edge(VertexClass *pu, VertexClass *pv) {
    Types::uint i;

    // add verticies to set as needed
    Types::uint iu = daVertex.ReplaceOrAppend(pu, pu);
    pu->i = iu;
    Types::uint iv = daVertex.ReplaceOrAppend(pv, pv);
    pv->i = iv;

    // add edge if needed
    DC_Edge_Base *pedge;
    bool fFound = false;
    for (i=0; i<daEdge.NumElem(); ++i) {
      pedge = daEdge[i];
      if (pedge->iu == iu && pedge->iv == iv) {
	fFound = true;
	break;
      }
    }
    if (!fFound) {
      pedge = new EdgeClass;
      pedge->sbName = StrDup("e");
      pedge->iu = iu;
      pedge->iv = iv;
      i = daEdge.Append(pedge);
      pedge->i = i;

      // prepend this edge to adjacency list for u
      DC_Edge_Base *pTmp = daVertex[iu]->pHead;
      daVertex[iu]->pHead = pedge;
      pedge->pNext = pTmp;
    }
    return i;
  };

  ostream &print(ostream &os) {
    os << "Graph{|V|=" << daVertex.NumElem()
       << ",|E|=" << daEdge.NumElem() << endl;
    for (Types::uint i=0; i<daVertex.NumElem(); ++i) {
      DC_Vertex_Base *pv = daVertex[i];
      os << "  " << *pv << " ->";
      DC_Edge_Base *pedge = pv->pHead;
      while (pedge) {
	os << ' ' << *daVertex[pedge->iv];
	if (pedge->fHidden) os << "(hidden)";
	pedge = pedge->pNext;
	if (pedge) os << ',';
      }
      os << endl;
    }
    return os << '}' << endl;
  };
};
template <class VertexClass, class EdgeClass>
ostream &operator<<(ostream &os,
		    DC_Graph_AdjacencyList_t<VertexClass, EdgeClass> &G) {
  return G.print(os);
};


#endif /* H_DIGRAPH */
// ****************************************************************************
