// -*- c++ -*-
// ****************************************************************************
//  Name: dc_heap.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 06/25/2001 : mdb - Created.
//        06/26/2001 : mdb - Fixed bug in insert().
//  Desc: Simple heap class.
// ****************************************************************************

#if !defined(H_HEAP)
#define H_HEAP

#include "lib/dc_standard.h"
#include "lib/dc_darray.h"


//#define DCDBG_HEAP_UPDATES

template <class KeyVal, class DataObj>
class DC_Heap {
protected:
  class HeapItem {
  public:
    KeyVal   key;
    DataObj *pData;
  };
  DArray<HeapItem> daHeap;

protected:
  // children of i are heaps, fix for i and down the heap as needed
  void heapify(unsigned int i) {
    unsigned int iMax;
    unsigned int iLeft = 2*i;
    unsigned int iRight = 2*i + 1;
    if (iLeft < daHeap.NumElem() && daHeap[iLeft].key > daHeap[i].key) {
      iMax = iLeft;
    } else {
      iMax = i;
    }
    if (iRight < daHeap.NumElem() && daHeap[iRight].key > daHeap[iMax].key) {
      iMax = iRight;
    }
    if (iMax != i) {
      HeapItem itemTmp = daHeap[i];
      daHeap[i] = daHeap[iMax];
      daHeap[iMax] = itemTmp;
      heapify(iMax);
    }
  };

public:
  DC_Heap(void) {
    HeapItem itemDummy;
    daHeap.AppendByVal(itemDummy);  // for easy parent/left/right numbering
  };
  ~DC_Heap(void) {};

  inline int num(void) { return daHeap.NumElem(); };

#if defined(DCDBG_HEAP_UPDATES)
  void dump(void) {
    cout << "heap: ";
    for (unsigned int i=0; i<daHeap.NumElem(); ++i) {
      cout << '[' << i << "]=" << daHeap[i].key << ' ';
    }
    cout << endl;
  };
#endif /* DCDBG_HEAP_UPDATES */

  // add w/o preserving the heap property (for efficiency)
  void add(KeyVal key, DataObj *pData) {
    HeapItem item;
    item.key = key;
    item.pData = pData;
    daHeap.AppendByVal(item);
#if defined(DCDBG_HEAP_UPDATES)
    cout << "DC_Heap::add(): done add " << key << endl; dump();
#endif /* DCDBG_HEAP_UPDATES */
  };

  // create a heap after multiple add() calls
  void build(void) {
    for (unsigned int i=(daHeap.NumElem()-1)/2; i>0; --i) heapify(i);
#if defined(DCDBG_HEAP_UPDATES)
    cout << "DC_Heap::build(): done" << endl; dump();
#endif /* DCDBG_HEAP_UPDATES */
  };

  // add and preserve the heap property
  void insert(KeyVal key, DataObj *pData) {
    unsigned int i;
    HeapItem item;
    daHeap.AppendByVal(item);  // add dummy placeholder
    for (i = daHeap.NumElem()-1; i>1 && daHeap[i/2].key < key; i = i/2) {
      daHeap[i] = daHeap[i/2];
    }
    item.key = key;
    item.pData = pData;
    daHeap[i] = item;
#if defined(DCDBG_HEAP_UPDATES)
    cout << "DC_Heap::insert(): done insert " << key << endl; dump();
#endif /* DCDBG_HEAP_UPDATES */
  };

  // remove the max value, and fixup the heap
  DataObj *extract_max(KeyVal *pKeyMax=NULL) {
    if (daHeap.NumElem() < 2) return NULL;
    DataObj *pData = daHeap[1].pData;
    if (pKeyMax) *pKeyMax = daHeap[1].key; 
    daHeap[1] = daHeap[daHeap.NumElem()-1];
    daHeap.Remove(daHeap.NumElem()-1);
    heapify(1);
#if defined(DCDBG_HEAP_UPDATES)
    cout << "DC_Heap::extract_max(): done extract" << endl; dump();
#endif /* DCDBG_HEAP_UPDATES */
    return pData;
  };
};


#endif /* H_HEAP */
// ****************************************************************************
