// ****************************************************************************
//  Name: dc_template-instantiation.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 04/23/2001 : mdb - Created.
//  Desc: centralized instantiation of templates needed for this directory.
// ****************************************************************************

#include "lib/dc_standard.h"


//
// This is one model of template instantiation -- all templates we
// need for the current subdirectory are instantiated here.  The
// obvious problem is template args cannot be static to a .cpp file.
// If any c++ compiler ever supports the template export keyword, this
// could all be easier.  To reduce code bloat, we instantiate all
// needed templates here, and tell the compiler to not do it
// automatically.
//
#if defined(HAVE_INSTANTIATIONS)

#include "dc_filterservice.h"

// for consoleprocessstate.cpp ...
#include "consoleprocessstate.h"
template class DArray<DC_FilterInstance_t*>;    // instantiate
template class DArray<DC_ConsoleWork*>;    	// instantiate
template class DArray<DC_PlacementSpec*>;  	// instantiate

// for dc_instance.cpp ...
#include "dc_instance.h"
template class DArray<DC_ConsoleFilterSpec*>;    // instantiate
template class DArray<DC_ConsoleStreamSpec*>;    // instantiate

// for remoteprocessstate.cpp ...
#include "remoteprocessstate.h"
template class DArray<DC_RemoteInStreamShared*>;  // instantiate
template class DArray<DC_RemoteInStreamCopy*>;    // instantiate
template class DArray<DC_RemoteOutStreamShared*>; // instantiate
template class DArray<DC_RemoteOutStreamCopy*>;   // instantiate
template class DArray<DC_RemoteFilterCopy*>;      // instantiate
template class DArray<DC_RemotePending*>;         // instantiate
template class DArray<DC_RemoteFilterShared*>;    // instantiate

// for streambufferqueue.cpp ...
#include "streambufferqueue.h"
template class DArray<DC_StreamBufferQueue::RemoteDisabled>;  // instantiate
template class SLList<DC_StreamBufferQueue::RemoteDisabled>;  // instantiate
#include "lib/dc_queue.h"
#include "lib/dc_queue.cpp"
template class DC_QueueT<DC_StreamBufferQueue_Item>;

#endif /* HAVE_INSTANTIATIONS */


// ****************************************************************************
