#include <iostream.h>
#include "dc_ctojava_filter_t.h"
#include "util/dc_jnimap.h"
#include "dc_register_methods.h"

#include "lib/dc_timer.h"

/*
 * Create an instance of the appropriate filter in java
 */
DC_Filter_Base_t *DC_CToJava_Filter_t::filter_factory(const char *sbFilterName) {
   jobject javaFilter;
   Timer t;

   cerr << "Got a filter factory call to create a filter" << endl;

   t.Start();
   if (NativeReg::ConstructNReg()->CreateJavaClass(NULL, sbFilterName, "()V", NULL, javaFilter) == 1) {
      cout << "Time taken by create class " << t.Elapsed() << endl;
      return new DC_CToJava_Filter_t(javaFilter);
   }
   else {
      cerr << "app: unknown filter \"" << sbFilterName << "\" to create" << endl;
   }
   return NULL;
}

/*
 * Find the appropriate java class and ensure proper native functions exist
 */
DC_CToJava_Filter_t::DC_CToJava_Filter_t(jobject newJavaFilter) {
   javaFilter = newJavaFilter;
   Timer t;

   t.Start();
   DC_JNIMap::GetJNIMap()->addPair(NULL, javaFilter, reinterpret_cast<jlong>(this));
   cout << "Time taken by addPair " << t.Elapsed() << endl;
   DC_RegisterMethods::registerMethods();
}

DC_CToJava_Filter_t::~DC_CToJava_Filter_t() {
}

/*
 * Call the appropriate function on the Java filter (forward call)
 */
DC_RTN_t DC_CToJava_Filter_t::init(initarg_t &arg) {
   jobject javaArg;
   Timer t;
   
   t.Start();
   JNIEnv *env = NativeReg::ConstructNReg()->GetEnv();

   NativeReg::ConstructNReg()->CreateJavaClass(env, "dc/jfilters/initarg_t", "()V", NULL, javaArg);
   DC_JNIMap::GetJNIMap()->addPair(env, javaArg, reinterpret_cast<jlong>(&arg));
   jvalue result = NativeReg::ConstructNReg()->InvokeMethod(javaFilter, env->GetObjectClass(javaFilter), "start_work", "(Ldc/jfilters/initarg_t;)I", javaArg);
   
   cout << "The time taken for a init call: " << t.Elapsed() << endl;
   return static_cast<DC_RTN_t>(result.i);
}

DC_RTN_t DC_CToJava_Filter_t::process(arg_t &arg) {
   jobject javaArg;
   Timer t;

   t.Reset();
   t.Start();
   JNIEnv *env = NativeReg::ConstructNReg()->GetEnv();

   NativeReg::ConstructNReg()->CreateJavaClass(env, "dc/jfilters/arg_t", "()V", NULL, javaArg);
   cout << "Time taken to create arg_t " << t.Elapsed() << endl;

   DC_JNIMap::GetJNIMap()->addPair(env, javaArg, reinterpret_cast<jlong>(&arg));
   cout << "Time taken by addPair " << t.Elapsed() << endl;

   jvalue result = NativeReg::ConstructNReg()->InvokeMethod(javaFilter, env->GetObjectClass(javaFilter), "process", "(Ldc/jfilters/arg_t;)I", javaArg);

   cout << "The time taken for a process call: " << t.Elapsed() << endl;
   return static_cast<DC_RTN_t>(result.i);
}

DC_RTN_t DC_CToJava_Filter_t::finalize(void) {
   Timer t;

   t.Start();
   JNIEnv *env = NativeReg::ConstructNReg()->GetEnv();

   jvalue result = NativeReg::ConstructNReg()->InvokeMethod(javaFilter, env->GetObjectClass(javaFilter), "finish_work", "()I");

   cout << "The time taken for a finalize call: " << t.Elapsed() << endl;
   return static_cast<DC_RTN_t>(result.i);
}
