/* 
* This file provides the native implementation of the native methods defined 
* in DC_Buffer_t.java 
*/

#include <jni.h>
#include "dc_jfilters_DC_0005fBuffer_0005ft.h"
#include "util/dc_jnimap.h"
#include "reg/NativeReg.h"
#include "dc_buffer.h"

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _new
* Signature: (J)[B
*/
JNIEXPORT jbyteArray JNICALL Java_dc_jfilters_DC_1Buffer_1t__1new
(JNIEnv *env, jobject thisJ, jlong wMax) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   jbyteArray result = env->NewByteArray(wMax);
   env->SetByteArrayRegion(result, 0, static_cast<Types::uint>(wMax), reinterpret_cast<jbyte *>(thisC->New(wMax)));
   
   return result;
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _delete_funct
* Signature: ()V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1delete_1funct
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   thisC->Delete();
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _set
* Signature: ([BII)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1set
(JNIEnv *env, jobject thisJ, jbyteArray pBuf_in, jint wMax, jint wSize, jboolean fFree) {
   jboolean isCopy;
   
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   jbyte* pBuf = env->GetByteArrayElements(pBuf_in, &isCopy);
   jbyte *pBufLocal = pBuf;

   if (isCopy == JNI_FALSE) {
      jbyte* pBufLocal = new jbyte[env->GetArrayLength(pBuf_in)];
      memcpy(reinterpret_cast<void *>(pBufLocal), reinterpret_cast<void *>(pBuf), static_cast<int>(env->GetArrayLength(pBuf_in) * sizeof(jbyte)));
   }

   //call set function
   thisC->Set(reinterpret_cast<char *>(pBufLocal), static_cast<Types::uint>(wMax),static_cast<Types::uint>(wSize), static_cast<bool>(fFree));

   //array isn't released because Mike does not make a copy of passed in char *
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _empty
* Signature: ()V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1empty
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   thisC->Empty();
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _getMax
* Signature: ()J
*/
JNIEXPORT jlong JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getMax
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jlong>(thisC->getMax());
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _getBuffer
* Signature: ()[B
*/
JNIEXPORT jbyteArray JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getBuffer
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));

   jbyteArray buffer = env->NewByteArray(thisC->getMax());

   env->SetByteArrayRegion(buffer, 0, thisC->getMax(), reinterpret_cast<jbyte *>(thisC->getPtr()));
   
   return buffer;
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _getSize
* Signature: ()J
*/
JNIEXPORT jlong JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getSize
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jlong>(thisC->getSize());
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _getFree
* Signature: ()J
*/
JNIEXPORT jlong JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getFree
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jlong>(thisC->getFree());
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _getFreeBuffer
* Signature: ()[B
*/
JNIEXPORT jbyteArray JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getFreeBuffer
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   jbyteArray buffer = env->NewByteArray(thisC->getFree());
   env->SetByteArrayRegion(buffer, 0, thisC->getFree(), reinterpret_cast<jbyte *>(thisC->getPtrFree()));
   
   return buffer;
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _getExtractBuffer
* Signature: ()[B
*/
JNIEXPORT jbyteArray JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getExtractBuffer
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   jbyteArray buffer = env->NewByteArray(thisC->getSizeExtract());
   env->SetByteArrayRegion(buffer, 0, thisC->getSizeExtract(), reinterpret_cast<jbyte *>(thisC->getPtrExtract()));
   
   return buffer;
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _getSizeExtract
* Signature: ()J
*/
JNIEXPORT jlong JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getSizeExtract
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jlong>(thisC->getSizeExtract());
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _resetExtract
* Signature: (J)I
*/
JNIEXPORT jlong JNICALL Java_dc_jfilters_DC_1Buffer_1t__1resetExtract
(JNIEnv *env, jobject thisJ, jlong w) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jlong>(thisC->resetExtract(static_cast<Types::uint>(w)));
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _append
* Signature: ([BJ)[B
*/
JNIEXPORT jbyteArray JNICALL Java_dc_jfilters_DC_1Buffer_1t__1append
(JNIEnv *env, jobject thisJ, jbyteArray p, jlong size) {
   jboolean isCopy;
   
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   Types::uint sizeC = static_cast<Types::uint>(size);
   jbyte* pC = env->GetByteArrayElements(p, &isCopy);

//   jbyteArray result = env->NewByteArray(thisC->getMax());
   reinterpret_cast<jbyte *>(thisC->Append(reinterpret_cast<char *>(pC), sizeC));

//   env->SetByteArrayRegion(result, 0, thisC->getMax(), resultC);
   
   //release the array in C++ but don't modify the original
   if (isCopy == JNI_TRUE) {
      env->ReleaseByteArrayElements(p, pC, JNI_ABORT);
   }

   return NULL;
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _extract
* Signature: ([BJ)I
*/
JNIEXPORT jint JNICALL Java_dc_jfilters_DC_1Buffer_1t__1extract
(JNIEnv *env , jobject thisJ, jbyteArray p, jlong size) {
   jboolean isCopy;
   
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   Types::uint sizeC = static_cast<Types::uint>(size);
   jbyte* pC = env->GetByteArrayElements(p, &isCopy);
   
   jint result = static_cast<int>(thisC->Extract(reinterpret_cast<char *>(pC), sizeC));
   
   //release the array in C++ and save any changes made to it
   if (isCopy == JNI_TRUE) {
      env->ReleaseByteArrayElements(p, pC, 0);
   }
   
   return result;
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _setSize
* Signature: (J)I
*/
JNIEXPORT jint JNICALL Java_dc_jfilters_DC_1Buffer_1t__1setSize
(JNIEnv *env, jobject thisJ, jlong wSize) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   Types::uint wSizeC = static_cast<Types::uint>(wSize);
   
   jint result = static_cast<jint>(thisC->setSize(wSizeC));
   return result;
}

/*
 * Class:     dc_jfilters_DC_0005fBuffer_0005ft
 * Method:    _getPNext
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getPNext
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t* thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));

   //create a reference between the java class and the C++ class
   if (thisC->pNext != NULL) {
      jfieldID javaContainerID = env->GetFieldID(env->GetObjectClass(thisJ), "pNext", "Ldc/jfilters/DC_Buffer_t;");
      if (javaContainerID) {
         jobject javaContainer = env->GetObjectField(thisJ, javaContainerID);
         if (javaContainer == NULL) {
	    //indicate to the java class not to create a C++ instance since it will be reassigned
	    jvalue params[1];
	    params[0].z = false;

            if (NativeReg::ConstructNReg()->CreateJavaClass(env, "dc/jfilters/DC_Buffer_t", "(Z)V", params,javaContainer)) {
               env->SetObjectField(thisJ, javaContainerID, javaContainer);
	    }
	    else {
		return;
	    }
         }
         DC_JNIMap::GetJNIMap()->addPairNoOverwrite(env, javaContainer, reinterpret_cast<jlong>(thisC->pNext));
      }
   }
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _setPNext
* Signature: (Ldc/jfilters/DC_Buffer_t;)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1setPNext
(JNIEnv *env, jobject thisJ, jobject pNext) {
   DC_Buffer_t* thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   //store the c++ class reference in thisJ class pNext
   thisC->pNext = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _initialize
* Signature: ()J
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1initialize__
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t* newClass = new DC_Buffer_t();
   
   DC_JNIMap::GetJNIMap()->addPair(env, thisJ, reinterpret_cast<jlong>(newClass));
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _initialize
* Signature: (I)J
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1initialize__I
(JNIEnv *env, jobject thisJ, jint wMax) {
   DC_Buffer_t *newClass = new DC_Buffer_t(wMax);
   
   DC_JNIMap::GetJNIMap()->addPair(env, thisJ, reinterpret_cast<jlong>(newClass));
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _clone
* Signature: (Ldc/jfilters/DC_Buffer_t;)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1clone
(JNIEnv *env, jobject thisJ, jobject clone) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   //make a c++ copy of the buffer
   DC_Buffer_t *cp = new DC_Buffer_t(*thisC);
   
   //store the clone's native reference to the copy
   DC_JNIMap::GetJNIMap()->addPair(env, clone, reinterpret_cast<jlong>(cp));
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _delete
* Signature: ()V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1delete
(JNIEnv *env, jobject thisJ) {
   DC_JNIMap::GetJNIMap()->remove(env, thisJ);
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _consume
* Signature: ()V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1consume
(JNIEnv *env , jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   thisC->consume();
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _getConsume
* Signature: ()Z
*/
JNIEXPORT jboolean JNICALL Java_dc_jfilters_DC_1Buffer_1t__1getConsume
(JNIEnv *env, jobject thisJ) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jboolean>(thisC->getConsume());
}

/*
* Class:     dc_jfilters_DC_0005fBuffer_0005ft
* Method:    _setConsume
* Signature: (Z)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Buffer_1t__1setConsume
(JNIEnv *env, jobject thisJ, jboolean newFConsume) {
   DC_Buffer_t *thisC = reinterpret_cast<DC_Buffer_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   thisC->setConsume(static_cast<bool>(newFConsume));
}
