/* 
* This file provides the native implementation of the native methods defined 
* in DC_Filter_Base_t.java 
*/

#include <jni.h>
#include <string.h>
#include "dc_jfilters_DC_0005fFilter_0005fBase_0005ft.h"
#include "util/dc_jnimap.h"
#include "dc_filterservice.h"

/*
* Class:     dc_jfilters_DC_0005fFilter_0005fBase_0005ft
* Method:    _getSbFilterName
* Signature: ()Ljava/lang/String;
*/
JNIEXPORT jstring JNICALL Java_dc_jfilters_DC_1Filter_1Base_1t__1getSbFilterName
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t *thisC = reinterpret_cast<DC_Filter_Base_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));

   jstring result = env->NewStringUTF(const_cast<const char *>(thisC->sbFilterName));

   return result;
}

/*
* Class:     dc_jfilters_DC_0005fFilter_0005fBase_0005ft
* Method:    _setSbFilterName
* Signature: (Ljava/lang/String;)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Filter_1Base_1t__1setSbFilterName
(JNIEnv *env, jobject thisJ, jstring newSbFilterName) {
   jboolean isCopy;
   DC_Filter_Base_t *thisC = reinterpret_cast<DC_Filter_Base_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   const char *newSbFilterNameC = env->GetStringUTFChars(newSbFilterName, &isCopy);

   if (thisC->sbFilterName != NULL) {
        delete[] thisC->sbFilterName;
   }

   thisC->sbFilterName = new char[env->GetStringUTFLength(newSbFilterName) + 1];
   thisC->sbFilterName = newSbFilterNameC;
   
   if (isCopy == JNI_TRUE) {
      env->ReleaseStringUTFChars(newSbFilterName, newSbFilterNameC);
   }
	cout << "Set filter name to: " << thisC->sbFilterName << endl;
}

/*
* Class:     dc_jfilters_DC_0005fFilter_0005fBase_0005ft
* Method:    _findAppFilterConfig
* Signature: (Ljava/lang/String;I)Ljava/lang/String;
*/
JNIEXPORT jstring JNICALL Java_dc_jfilters_DC_1Filter_1Base_1t__1findAppFilterConfig
(JNIEnv *env, jobject thisJ, jstring sbName, jint wId) {
   jboolean isCopy;
   
   DC_Filter_Base_t *thisC = reinterpret_cast<DC_Filter_Base_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   const char *sbNameC = env->GetStringUTFChars(sbName, &isCopy);
   
   const char* result = const_cast<const char *>(thisC->FindAppFilterConfig(const_cast<char *>(sbNameC), static_cast<int>(wId)));
   
   if (isCopy == JNI_TRUE) {
      env->ReleaseStringUTFChars(sbName, sbNameC);
   } 
   
   return env->NewStringUTF(result);
}

/*
* Class:     dc_jfilters_DC_0005fFilter_0005fBase_0005ft
* Method:    _findAppFilterConfigInt
* Signature: (Ljava/lang/String;IZ)J
*/
JNIEXPORT jlong JNICALL Java_dc_jfilters_DC_1Filter_1Base_1t__1findAppFilterConfigInt
(JNIEnv *env, jobject thisJ, jstring sbName, jint wId, jboolean fInterpretUnits) {
   jboolean isCopy;
   
   DC_Filter_Base_t *thisC = reinterpret_cast<DC_Filter_Base_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   const char *sbNameC = env->GetStringUTFChars(sbName, &isCopy);
   
   jlong result = static_cast<jlong>(thisC->FindAppFilterConfigInt(const_cast<char *>(sbNameC), static_cast<int>(wId), static_cast<bool>(fInterpretUnits)));
   
   if (isCopy == JNI_TRUE) {
      env->ReleaseStringUTFChars(sbName, sbNameC);
   }     
   
   return result;
}

/*
* Class:     dc_jfilters_DC_0005fFilter_0005fBase_0005ft
* Method:    _findAppFilterConfigDbl
* Signature: (Ljava/lang/String;I)D
*/
JNIEXPORT jdouble JNICALL Java_dc_jfilters_DC_1Filter_1Base_1t__1findAppFilterConfigDbl
(JNIEnv *env , jobject thisJ, jstring sbName, jint wId) {
   jboolean isCopy;
   
   DC_Filter_Base_t *thisC = reinterpret_cast<DC_Filter_Base_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   const char *sbNameC = env->GetStringUTFChars(sbName, &isCopy);
   
   jdouble result = static_cast<jdouble>(thisC->FindAppFilterConfigDbl(const_cast<char *>(sbNameC), static_cast<int>(wId)));
   
   if (isCopy == JNI_TRUE) {
      env->ReleaseStringUTFChars(sbName, sbNameC);
   }     
   
   return result;
}

/*
* Class:     dc_jfilters_DC_0005fFilter_0005fBase_0005ft
* Method:    _filterLock
* Signature: ()V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Filter_1Base_1t__1filterLock
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t *thisC = reinterpret_cast<DC_Filter_Base_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   thisC->FilterLock();
}

/*
* Class:     dc_jfilters_DC_0005fFilter_0005fBase_0005ft
* Method:    _filterUnlock
* Signature: ()V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_DC_1Filter_1Base_1t__1filterUnlock
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t *thisC = reinterpret_cast<DC_Filter_Base_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   thisC->FilterUnlock();
}
