/* 
* This file provides the native implementation of the native methods defined 
* in DC_Work_t.java 
*/

#include <jni.h>
#include <string.h>
#include "dc_jfilters_initarg_0005ft.h"
#include "util/dc_jnimap.h"
#include "reg/NativeReg.h"
#include "dc_filterservice.h"

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _getArgc
* Signature: ()I
*/
JNIEXPORT jint JNICALL Java_dc_jfilters_initarg_1t__1getArgc
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jint>(thisC->argc);
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _setArgc
* Signature: (I)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_initarg_1t__1setArgc
(JNIEnv *env, jobject thisJ, jint newArgc) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   thisC->argc = static_cast<int>(newArgc);
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _getArgv
* Signature: ()[Ljava/lang/String;
*/
JNIEXPORT jobjectArray JNICALL Java_dc_jfilters_initarg_1t__1getArgv__
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   //create an array to hold the argv data
   jobjectArray result = env->NewObjectArray(thisC->argc, env->FindClass("java/lang/String"), NULL);
   
   for (int i = 0; i < thisC->argc; i++) {
      env->SetObjectArrayElement(result, i, env->NewStringUTF(const_cast<const char *>(thisC->argv[i])));
   }
   
   return result;
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _getArgv
* Signature: (I)Ljava/lang/String;
*/
JNIEXPORT jstring JNICALL Java_dc_jfilters_initarg_1t__1getArgv__I
(JNIEnv *env, jobject thisJ, jint index) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   jstring result = env->NewStringUTF(const_cast<const char *>(thisC->argv[static_cast<int>(index)]));
   
   return result;
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _setArgv
* Signature: ([Ljava/lang/String;)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_initarg_1t__1setArgv___3Ljava_lang_String_2
(JNIEnv *env, jobject thisJ, jobjectArray newArgv) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   //reallocate a new argv
   if (thisC->argv != NULL) {
      delete[] thisC->argv;
   }
   thisC->argv = new char *[env->GetArrayLength(newArgv)];
   
   //copy the Strings from newArgv into C++ char *s
   for (int i = 0; i < thisC->argc; i++) {
      jboolean isCopy;
      
      //access current string
      jstring curr = reinterpret_cast<jstring>(env->GetObjectArrayElement(newArgv, i));
      const char *currC = env->GetStringUTFChars(curr, &isCopy);
      
      //copy the current string into the correct location of thisC->argv
      thisC->argv[i] = new char[env->GetStringUTFLength(curr) + 1];
      strcpy(thisC->argv[i], currC);
      
      if (isCopy == JNI_TRUE) {
         env->ReleaseStringUTFChars(curr, currC);
      }
      
      //release the extra memory used by current string
      env->DeleteLocalRef(curr);
   }
   
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _setArgv
* Signature: (Ljava/lang/String;I)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_initarg_1t__1setArgv__Ljava_lang_String_2I
(JNIEnv *env, jobject thisJ, jstring newArgv, jint index) {
   jboolean isCopy;
   
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   int indexC = static_cast<int>(index);
   
   //delete old argv at location index
   if (thisC->argv[indexC] != NULL) {
      delete[] thisC->argv[indexC];
   }
   thisC->argv[indexC] = new char[env->GetStringLength(newArgv) + 1];
   
   //store the newArgvC in the appropriate location of thisC->argv
   const char *newArgvC = env->GetStringUTFChars(newArgv, &isCopy);
   strcpy(thisC->argv[indexC], newArgvC);
   
   if (isCopy == JNI_TRUE) {
      env->ReleaseStringUTFChars(newArgv, newArgvC);
   }
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _getSbFilterName
* Signature: ()Ljava/lang/String;
*/
JNIEXPORT jstring JNICALL Java_dc_jfilters_initarg_1t__1getSbFilterName
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));

   jstring result = env->NewStringUTF(const_cast<const char *>(thisC->sbFilterName));
   return result;
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _setSbFilterName
* Signature: (Ljava/lang/String;)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_initarg_1t__1setSbFilterName
(JNIEnv *env, jobject thisJ, jstring newSbFilterName) {
   jboolean isCopy;
   
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   if (thisC->sbFilterName != NULL) {
      delete[] thisC->sbFilterName;
   }
   
   const char *newSbFilterNameC = env->GetStringUTFChars(newSbFilterName, &isCopy);

   thisC->sbFilterName = new char[env->GetStringLength(newSbFilterName) + 1];
   thisC->sbFilterName = newSbFilterNameC;
   
   if (isCopy == JNI_TRUE) {
      env->ReleaseStringUTFChars(newSbFilterName, newSbFilterNameC);
   }
}

/*
 * Class:     dc_jfilters_initarg_0005ft
 * Method:    _getPWork
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_dc_jfilters_initarg_1t__1getPWork
(JNIEnv *env, jobject thisJ) {
  DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));

   //create a reference between the java class and the C++ class
   if (thisC->pwork != NULL) {

      jfieldID javaContainerID = env->GetFieldID(env->GetObjectClass(thisJ), "pWork", "Ldc/jfilters/DC_Work_t;");
 
      if (javaContainerID) {
         jobject javaContainer = env->GetObjectField(thisJ, javaContainerID);
         if (javaContainer == NULL) {
            if (NativeReg::ConstructNReg()->CreateJavaClass(env, "dc/jfilters/DC_Work_t", "()V", NULL, javaContainer)) {
               env->SetObjectField(thisJ, javaContainerID, javaContainer);
            }
         }
	 cout << "Adding pair link for pWork in initarg: " << reinterpret_cast<jlong>(thisC->pwork) << endl;
         DC_JNIMap::GetJNIMap()->addPair(env, javaContainer, reinterpret_cast<jlong>(thisC->pwork));
      }
   }
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _setPWork
* Signature: (Ldc/jfilters/DC_Work_t;)V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_initarg_1t__1setPWork
(JNIEnv *env, jobject thisJ, jobject newPWork) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   //store the c++ class reference in this class pwork
   thisC->pwork = reinterpret_cast<DC_Work_t *>(DC_JNIMap::GetJNIMap()->find(env, newPWork));
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _initialize
* Signature: ()J
*/
JNIEXPORT void JNICALL Java_dc_jfilters_initarg_1t__1initialize
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t::initarg_t* newClass = new DC_Filter_Base_t::initarg_t();
   
   DC_JNIMap::GetJNIMap()->addPair(env, thisJ, reinterpret_cast<jlong>(newClass));
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _delete
* Signature: ()V
*/
JNIEXPORT void JNICALL Java_dc_jfilters_initarg_1t__1delete
(JNIEnv *env, jobject thisJ) {
   DC_JNIMap::GetJNIMap()->remove(env, thisJ);
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _maxCopySets
* Signature: ()I
*/
JNIEXPORT jint JNICALL Java_dc_jfilters_initarg_1t__1maxCopySets
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jint>(thisC->maxCopySets());
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _rankCopySets
* Signature: ()I
*/
JNIEXPORT jint JNICALL Java_dc_jfilters_initarg_1t__1rankCopySets
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jint>(thisC->rankCopySets());
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _maxCopies
* Signature: ()I
*/
JNIEXPORT jint JNICALL Java_dc_jfilters_initarg_1t__1maxCopies
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jint>(thisC->maxCopies());
}

/*
* Class:     dc_jfilters_initarg_0005ft
* Method:    _rankCopies
* Signature: ()I
*/
JNIEXPORT jint JNICALL Java_dc_jfilters_initarg_1t__1rankCopies
(JNIEnv *env, jobject thisJ) {
   DC_Filter_Base_t::initarg_t *thisC = reinterpret_cast<DC_Filter_Base_t::initarg_t *>(DC_JNIMap::GetJNIMap()->find(env, thisJ));
   
   return static_cast<jint>(thisC->rankCopies());
}
