#include "NativeReg.h"
#include <iostream.h>
#include <string>

FILE      *NativeReg::m_fp = 0;
NativeReg *NativeReg::m_this = 0;
DC_Mutex  *NativeReg::lock = new DC_Mutex();

NativeReg* NativeReg::ConstructNReg() {
  JavaVMAttachArgs vmArgs;
  lock->lock();
  if (!NativeReg::m_this) {
    NativeReg::m_this = new NativeReg;
  }
  //bind to the current thread if necessary
  if (NativeReg::m_this->m_pjvm->GetEnv((void **)&NativeReg::m_this->m_penv, JNI_VERSION_1_2) == JNI_EDETACHED) {
    vmArgs.version = JNI_VERSION_1_2;
    vmArgs.name = NULL;
    vmArgs.group = NULL;
    NativeReg::m_this->m_pjvm->AttachCurrentThread((void **)&NativeReg::m_this->m_penv, (void*)&vmArgs);
  }
  
  lock->unlock();
  return NativeReg::m_this;
}

NativeReg::NativeReg() {
  JavaVMInitArgs vmArgs;
  JavaVMOption   options[3];
  
  //JNI_GetDefaultJavaVMInitArgs(&vmArgs);
  //set up the JVM options
  options[0].optionString = "vfprintf";
  options[0].extraInfo = (void *)n_fprintf;
  options[1].optionString = USER_CLASSPATH;
  options[2].optionString = "-verbose:class,gc,jni";
  
  cout << "Set up options" << endl;
  
  //set up the JVM args, see jni documentation for possible setup
  vmArgs.version = JNI_VERSION_1_2;
  vmArgs.options = options;
  vmArgs.nOptions = 3;
  vmArgs.ignoreUnrecognized = JNI_TRUE;
  
  cout << "Set up ARGS" << endl;
  
  //create a JVM
  jint result = JNI_CreateJavaVM(&m_pjvm, (void**)&m_penv, (void*)&vmArgs);
  
  // CLH - added more informative error reports
  cout << "Create JVM: ";
  switch(result) {
  case JNI_OK:
    cout << "success";
    break;
  case JNI_ERR: 
    cout << "unknown error";
    break;
  case JNI_EDETACHED:
    cout << "thread detached from the VM";
    break;
  case JNI_EVERSION:
    cout << "JNI version error";
    break;
  case JNI_ENOMEM:
    cout << "not enough memory";
    break;
  case JNI_EEXIST:
    cout << "VM already created";
    break;
  case JNI_EINVAL:
    cout << "invalid arguments";
    break;
  }
  cout << endl;
}

NativeReg::~NativeReg() {
  std::vector<FParams*>::iterator iter;
  // close the JVM
  if (m_pjvm) {
    // unregister natives
    for(iter = m_fvect.begin(); iter != m_fvect.end(); iter++) {
      jclass clazz;
      
      clazz = (*iter)->clazz;
      m_penv->UnregisterNatives(clazz);
    }
    m_pjvm->DestroyJavaVM();
  }
  if (m_fvect.size()) { // delete all FParam structures that have been allocated
      for(; m_fvect.size(); ) {
	iter = m_fvect.begin();
	delete *iter;
	m_fvect.erase(m_fvect.begin());
      }
  }
  delete lock;
}

jclass NativeReg::RegisterMethod(char *class_name, char *method_name, char *signature, void *fptr) {
  jclass clazz;
  
  lock->lock();
  
  m_penv->ExceptionDescribe();
  
  lock->unlock();
  clazz = m_penv->FindClass(class_name);
  lock->lock();    
  
  m_penv->ExceptionDescribe();
  
  if (clazz) {
    FParams         *fparam = new FParams;
    JNINativeMethod  method;
    
    fparam->fptr = fptr;
    fparam->fsignature = signature;
    fparam->fname = method_name;
    fparam->clazz = clazz; 
    
    method.name = (char*)fparam->fname.c_str();
    method.signature = (char*)fparam->fsignature.c_str();
    method.fnPtr = fptr;
    // and now register the native method with the JVM
    if (!m_penv->RegisterNatives(clazz, &method, 1)) {
      // save the array that is the signature for this class so that during invokation
      // one does not need to get the signature of the method
      SigToVector(signature, fparam->arg_vec);
      m_fvect.push_back(fparam);
      
      lock->unlock();
      return clazz;
    } else {
      lock->unlock();
      return 0; // failed to register
    }
  } else {
    lock->unlock();
    return 0;
  }
}

int NativeReg::GetNativeCount() {
  return m_fvect.size();
}

// need the jc and the method_name
// can't use method_name alone as couple of classes might have the same method name
jvalue NativeReg::InvokeRegMethod(jobject &obj, jclass jc, char *method_name, ...) {
  // look up the class_name and the method name
  std::vector<FParams*>::iterator iter;
  jvalue                          res;
  
  lock->lock();
  
  memset(&res, 0, sizeof(jvalue));
  for(iter = m_fvect.begin(); iter != m_fvect.end(); iter++) {
    if ((*iter)->fname == method_name && (*iter)->clazz == jc) {
      jmethodID mid;
      // found the correct method and class
      // time to prepare the args and send them
      mid = m_penv->GetMethodID(jc, (*iter)->fname.c_str(), (*iter)->fsignature.c_str());
      if (mid) {
	jvalue                       *args;
	va_list                       vlist;
	std::vector<FArgs>::iterator  arg_iter;
	unsigned int                  i;
	
	va_start(vlist, method_name);
	
	args = new jvalue[(*iter)->arg_vec.size() - 1]; // last one is the return type
	for(arg_iter = (*iter)->arg_vec.begin(), i = 0; i < (*iter)->arg_vec.size() - 1; arg_iter++, i++) {
	  memset(args + i, 0, sizeof(jvalue));
	  // needs to support more types
	  switch(arg_iter->type) {
	  case 'B':   args[i].b = (jbyte)va_arg(vlist, int);
	    break;
	  case 'C':   args[i].c = (jchar)va_arg(vlist, int);
	    break;
	  case 'D':   args[i].d = (jdouble)va_arg(vlist, double);
	    break;
	  case 'F':   args[i].f = (jfloat)va_arg(vlist, double);
	    break;
	  case 'I':   args[i].i = (jint)va_arg(vlist, int);
	    break;
	  case 'J':   args[i].j = (jlong)va_arg(vlist, long);
	    break;
	  case 'S':   args[i].s = (jshort)va_arg(vlist, int);
	    break;
	  case 'Z':   args[i].z = (jboolean)va_arg(vlist, int);
	    break;
	  case 'O':    // handle arrays or objects here
	    args[i].l = (jobject)va_arg(vlist, jobject);
	    break;
	  } 
	}
	va_end(vlist);
	// now need to find out what return value to expect.  last
	arg_iter = (*iter)->arg_vec.begin() + (*iter)->arg_vec.size() - 1;
	
	lock->unlock();
	switch(arg_iter->type) {
	case 'V':   m_penv->CallVoidMethodA(obj, mid, args);
	  break;
	case 'B':   res.b = m_penv->CallByteMethodA(obj, mid, args);
	  break;
	case 'C':   res.c = m_penv->CallCharMethodA(obj, mid, args);
	  break;
	case 'D':   res.d = m_penv->CallDoubleMethodA(obj, mid, args);
	  break;
	case 'F':   res.f = m_penv->CallFloatMethodA(obj, mid, args);
	  break;
	case 'I':   res.i = m_penv->CallIntMethodA(obj, mid, args);
	  break;
	case 'J':   res.j = m_penv->CallLongMethodA(obj, mid, args);
	  break;
	case 'S':   res.s = m_penv->CallShortMethodA(obj, mid, args);
	  break;
	case 'Z':   res.z = m_penv->CallBooleanMethodA(obj, mid, args);
	  break;
	case 'O':   // handle arrays or objects here
	  res.l = m_penv->CallObjectMethodA(obj, mid, args);
	  break;
	}
	      
	delete[] args;
	return res;
      }
      else {
	return res;
      }      
    }
  }
  return res;
}

jvalue NativeReg::InvokeMethod(jobject &obj, jclass jc, char *method_name, char *method_sig ...) {
  jvalue    res;
  jmethodID mid;
  
  memset(&res, 0, sizeof(jvalue));
  
  lock->lock();
  
  // found the correct method and class
  // time to prepare the args and send them
  mid = m_penv->GetMethodID(jc, method_name, method_sig);
  if (mid) {
    std::vector<FArgs>            arg_vec;
    jvalue                       *args;
    va_list                       vlist;
    std::vector<FArgs>::iterator  arg_iter;
    unsigned int                           i;
    
    va_start(vlist, method_sig);
    
    SigToVector(method_sig, arg_vec);
    args = new jvalue[arg_vec.size() - 1]; // last one is the return type
    for(arg_iter = arg_vec.begin(), i = 0; i < arg_vec.size() - 1; arg_iter++, i++) {
      memset(args + i, 0, sizeof(jvalue));
      // needs to support more types
      switch(arg_iter->type) {
      case 'B':   args[i].b = (jbyte)va_arg(vlist, int);
	break;
      case 'C':   args[i].c = (jchar)va_arg(vlist, int);
	break;
      case 'D':   args[i].d = (jdouble)va_arg(vlist, double);
	break;
      case 'F':   args[i].f = (jfloat)va_arg(vlist, double);
	break;
      case 'I':   args[i].i = (jint)va_arg(vlist, int);
	break;
      case 'J':   args[i].j = (jlong)va_arg(vlist, long);
	break;
      case 'S':   args[i].s = (jshort)va_arg(vlist, int);
	break;
      case 'Z':   args[i].z = (jboolean)va_arg(vlist, int);
	break;
      case 'O':    // handle arrays or objects here
	args[i].l = (jobject)va_arg(vlist, jobject);
	break;
      }
    }
    va_end(vlist);
    // now need to find out what return value to expect.  last
    arg_iter = arg_vec.begin() + arg_vec.size() - 1;
    
    lock->unlock();
    switch(arg_iter->type) {
    case 'V':   m_penv->CallVoidMethodA(obj, mid, args);
      break;
    case 'B':   res.b = m_penv->CallByteMethodA(obj, mid, args);
      break;
    case 'C':   res.c = m_penv->CallCharMethodA(obj, mid, args);
      break;
    case 'D':   res.d = m_penv->CallDoubleMethodA(obj, mid, args);
      break;
    case 'F':   res.f = m_penv->CallFloatMethodA(obj, mid, args);
      break;
    case 'I':   res.i = m_penv->CallIntMethodA(obj, mid, args);
      break;
    case 'J':   res.j = m_penv->CallLongMethodA(obj, mid, args);
      break;
    case 'S':   res.s = m_penv->CallShortMethodA(obj, mid, args);
      break;
    case 'Z':   res.z = m_penv->CallBooleanMethodA(obj, mid, args);
      break;
    case 'O':   // handle arrays or objects here
      res.l = m_penv->CallObjectMethodA(obj, mid, args);
      break;
    }
    delete[] args;
    
    return res;
  }
  else {
    return res;
  }
  return res;
}

int NativeReg::CreateDefObject(jclass clazz, jobject &obj) {
  lock->lock();
  
  if (m_penv && clazz) {
    jmethodID mid;
    
    mid = m_penv->GetMethodID(clazz, "<init>", "()V");
    obj = m_penv->NewObject(clazz, mid, NULL);
    lock->unlock();
    return 1;
  }
  lock->unlock();
  return 0;
}

int NativeReg::CreateJavaClass(JNIEnv *env, const char *className, const char *constructorSig, jvalue params[], jobject &result) {
  JNIEnv *envToUse = m_penv;
  
  lock->lock();
  
  if (env != m_penv && env != NULL) {
    envToUse = env;
  }
  
  jclass JavaClass = envToUse->FindClass(className);
  
  if (JavaClass != NULL) {
    jmethodID JavaClassConst = envToUse->GetMethodID(JavaClass, "<init>", constructorSig);
    if (JavaClassConst != NULL) {
      lock->unlock();
      result = envToUse->NewObjectA(JavaClass, JavaClassConst, params);
      
      return 1;
    }
  }
  
  lock->unlock();
  return 0;
}

jint NativeReg::GetHashCode(JNIEnv *env, jobject obj) {
  jint hashCode = 0;
  JNIEnv *envToUse = m_penv;
  
  lock->lock();
  
  if (env != m_penv && env != NULL) {
    envToUse = env;
  }
  
  jclass JavaClass = envToUse->GetObjectClass(obj);
  if (JavaClass) {
    jmethodID hashCodeMID = envToUse->GetMethodID(JavaClass, "hashCode", "()I");
    if (hashCodeMID) {
      hashCode = envToUse->CallIntMethod(obj, hashCodeMID);
    }
  }
  
  lock->unlock();
  return hashCode;
}

JNIEnv *NativeReg::GetEnv() {
  return m_penv;
}

int NativeReg::SigToVector(char *signature, std::vector<FArgs> &vec) {
  int   len = strlen(signature);
  FArgs arg;
  
  for(int i = 0; i < len; i++) {
    if (signature[i] != '(' && signature[i] != ')') {
      arg.barr = false;
      arg.class_path = "";
      if (signature[i] == '[') { // array of some type
	arg.barr = true;
	i++;
      }
      if (signature[i] == 'L') { // fully-qualified class
	arg.type = 'O'; // object
	
	for(++i; i < len && signature[i] != ';'; i++) {
	  arg.class_path += signature[i];
	}
      }
      else { // regular arg
	arg.type = signature[i];
      }
      vec.push_back(arg);
    }
  }
  return 1;
}

// get an array of parameter types expected for the method
jobjectArray NativeReg::GetParameterTypes(jobject method) {
  jclass    clazz = m_penv->GetObjectClass(method);
  jmethodID mid = m_penv->GetMethodID(clazz, "getParameterTypes", "()[Ljava/lang/Class;");
  return (jobjectArray) m_penv->CallObjectMethod(method, mid, NULL);
}

// convert a java type into a C string name
const char*  NativeReg::ToString(jclass clazz) {
  jboolean  isCopy;
  jstring   jstr;
  jclass    classClz = m_penv->GetObjectClass(clazz);
  jmethodID mid = m_penv->GetMethodID(classClz, "toString", "()Ljava/lang/String;");
  
  jstr = (jstring)m_penv->CallObjectMethod(clazz, mid, 0);
  return m_penv->GetStringUTFChars(jstr, &isCopy);
}

// get return type from a method
jclass NativeReg::GetReturnType(jobject method) {
  jclass    clazz = m_penv->GetObjectClass(method);
  jmethodID mid = m_penv->GetMethodID(clazz, "getReturnType", "()Ljava/lang/class;");
  return (jclass)m_penv->CallObjectMethod(method, mid, NULL);
}

jobjectArray NativeReg::GetMethods(jclass Obj) {
  jclass    clazz = m_penv->GetObjectClass(Obj);
  jmethodID mid = m_penv->GetMethodID(clazz, "getMethods", "()[Ljava/lang/reflect/Method;");
  return (jobjectArray)m_penv->CallObjectMethod(Obj, mid, NULL);
}

jint (JNICALL NativeReg::n_fprintf) (FILE *, const char *format, va_list args) {
  jint rc;
  
  if (!NativeReg::m_fp) {
    if (!(NativeReg::m_fp = fopen("vm.out.txt", "w")))
      return 0;
  } 
  else {
    rc = vfprintf(NativeReg::m_fp, format, args);
    fflush(NativeReg::m_fp);
  }
  return rc;
}
