#ifndef NATIVE_REG_H
#define NATIVE_REG_H

#include <vector>
#include <string>
#include <jni.h>
#include "lib/dc_mutex.h"

//define for CLASSPATH from autoconf
#define USER_CLASSPATH "-Djava.class.path=/fs/schisto/chansen/work/DataCutter-v2/example/jfilters:/fs/schisto/chansen/work/DataCutter-v2/src/java"

class NativeReg
{
 private:
  struct FArgs
  {
    char        type; // type of the argument
    bool        barr; // array or just a single argument
    std::string class_path; // path of the class in case type is 'O' for object
  };
  
  struct FParams
  {
    jclass              clazz;      // class id
    std::string         fname;      // name of the function
    std::string         fsignature; // function signature in JAVA
    void               *fptr;       // function pointer
    std::vector<FArgs>  arg_vec;
  };
  
 private:
  NativeReg();
 public:
  ~NativeReg();
  
 public:
  jclass  RegisterMethod(char *class_name, char *method_name, char *signature, void *fptr);
  int     GetNativeCount();
  jvalue  InvokeRegMethod(jobject &obj, jclass jc, char *method_name, ...);
  jvalue  InvokeMethod(jobject &obj, jclass jc, char *method_name, char *method_sig ...);
  int     CreateDefObject(jclass clazz, jobject &obj);
  int     CreateJavaClass(JNIEnv *env, const char *className, const char *constructorSig, jvalue params[], jobject &result);
  jint    GetHashCode(JNIEnv *env, jobject obj);
  JNIEnv *GetEnv();
  
 private:
  static jint  (JNICALL n_fprintf) (FILE *, const char *format, va_list args);
  jobjectArray GetParameterTypes(jobject method);
  const char*  ToString(jclass clazz);
  jclass       GetReturnType(jobject method);
  jobjectArray GetMethods(jclass Obj);
  int          SigToVector(char *signature, std::vector<FArgs> &vec);
  
 public:
  static NativeReg* ConstructNReg();
  
 private:
  static NativeReg *m_this;
  JNIEnv           *m_penv;
  JavaVM           *m_pjvm;
  JDK1_1InitArgs    m_vm_args;
  static DC_Mutex	 *lock;
  
  static FILE      *m_fp;

  std::vector<FParams*> m_fvect; // function vector that contains all registered functions  
};

#endif
