/**
 * This is a utility map class based on the std::map provided in the stl
 * 
 * Purpose: Maintain connections between Java and corresponding C++ classes
*/

#if !defined(H_DC_JNIMAP)
#define H_DC_JNIMAP

#include <jni.h>
#include <map>
#include "lib/dc_mutex.h"

typedef std::map<jint, jlong> JINT2JLONG;

class DC_JNIMap  {

public:
   static DC_JNIMap* GetJNIMap();

public:
   //find if a C++ class exists for a given Java class
   jlong find(JNIEnv *env, jobject javaObject);

   //add a link between a Java Class and a C++ class
   void addPairNoOverwrite(JNIEnv *env, jobject javaObject, jlong cObject);
   void addPair(JNIEnv *env, jobject javaObject, jlong cObject);

   //remove a link between a C++ class and its java counterpart using the C++ class
   bool remove(JNIEnv *env, jobject javaObject);

   inline int getSize() { return javaToC->size(); }
   inline void clear() { javaToC->clear(); }

public:
   ~DC_JNIMap();
private:
   DC_JNIMap();
   static DC_Mutex *lock;
   static DC_JNIMap *instance;
   
private:
   JINT2JLONG *javaToC;
};

#endif // !defined(H_DC_JNIMAP)
