// -*- c++ -*-
// ****************************************************************************
//  Name: dc_argv.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 05/19/1997 : mdb - Created.
//  Desc: Utilities for manipulating argc argv pairs.
// ****************************************************************************
/** @file dc_argv.h */

#if !defined(H_ARGV)
#define H_ARGV

#include "dc_standard.h"
#include "dc_darray.h"
#include "dc_str.h"

#include <sys/types.h>


#if 0
typedef int argc_t;
typedef char **argv_t;
#endif

/// Utility for manipulating argc argv pairs.
class Argv {
protected:
  char   **rgElem;
  Types::uint    maxElem;
  Types::uint    nElem;
  bool *rgfDelete;

  void Extend(void);

public:
  Argv(void);
  ~Argv(void);

  void Set(int argc, char *argv[], bool fDelete = false);
  void Set(DArray<char*> da, bool fDelete);
  int Append(char *sbNewArg, bool fDelete = false);
  int Append(DArray<char*> da, bool fDelete = false);
  int AppendDup(char *sbNewArg) { return Append(StrDup(sbNewArg), true); };
  void Flush(void);
  
  int argc(void) { return nElem; };
  int GetNum(void) { return nElem; };
  char **argv(void) { return rgElem; };
  char *Get(Types::uint i) { return (i>maxElem) ? (char*)NULL : rgElem[i]; };
  char *operator[](Types::uint i) { return Get(i); };
  
  bool Find(char *sbFlag);
  char *NextFind(char *sbFlag);
  char *NextExtract(char *sbFlag);
  
  int toString(Str &strOut, bool fAddQuotes=true);

  friend ostream &operator<<(ostream &os, Argv &argv);
};


#endif /* H_ARGV */
// ****************************************************************************
