// ****************************************************************************
//  Name: dc_cond.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 11/20/2000 : mdb - Created.
//        02/27/2001 : mdb - Added internal calls to mutex.setOwner().
//        04/13/2001 : mdb - Changed debugging mutex calls.
//        04/23/2001 : mdb - Changed to use dc_pthread-wrappers.h
//  Desc: Simple synchronization wrappers.
// ****************************************************************************
 
#include <iostream>
using namespace std;

#include <errno.h>
#include <string.h>

#include "dc_cond.h"
#include "dc_pthread-wrappers.h"


//
//  class DC_Cond
//

// ctor/dtor
DC_Cond::DC_Cond(pthread_condattr_t *pattr) {
  int ret = Pthread_cond_init(&cv, pattr);
  if (ret != 0) {
    cerr << "DC_Cond(): failed pthread_cond_init - " << endl;
    // CLH - not MT safe - << strerror(errno) << endl;
  }
}

DC_Cond::~DC_Cond(void) {
  int ret = pthread_cond_destroy(&cv);
  if (ret != 0) {
    cerr << "DC_Cond(): failed pthread_cond_destroy - " << endl;
    // CLH - not MT safe - << strerror(errno) << endl;
  }
}

void DC_Cond::signal(void) {
  int ret = pthread_cond_signal(&cv);
  if (ret != 0) {
    cerr << "DC_Cond::signal() failed pthread_cond_signal - " << endl;
    // CLH - not MT safe - << strerror(errno) << endl;
  }  
}

void DC_Cond::broadcast(void) {
  int ret = pthread_cond_broadcast(&cv);
  if (ret != 0) {
    cerr << "DC_Cond::broadcast() failed pthread_cond_broadcast - " << endl;
    // CLH - not MT safe - << strerror(errno) << endl;
  }  
}

void DC_Cond::wait(DC_Mutex &mutex) {
#if defined(DCDBG_USE_MUTEX_CHECKS)
  mutex.implicitUnlock();
#endif /* DCDBG_USE_MUTEX_CHECKS */
  int ret = pthread_cond_wait(&cv, &mutex.getNative());
  if (ret != 0) {
    cerr << "DC_Cond::wait() failed pthread_cond_wait - " << endl;
    // CLH - not MT safe - << strerror(errno) << endl;
  }
#if defined(DCDBG_USE_MUTEX_CHECKS)
  else {
    mutex.implicitLock();
  }
#endif /* DCDBG_USE_MUTEX_CHECKS */
}

bool DC_Cond::wait(DC_Mutex &mutex, const struct timespec *abstime) {
#if defined(DCDBG_USE_MUTEX_CHECKS)
  mutex.implicitUnlock();
#endif /* DCDBG_USE_MUTEX_CHECKS */
  int ret = pthread_cond_timedwait(&cv, &mutex.getNative(), abstime);
  if (ret == ETIMEDOUT) {
#if defined(DCDBG_USE_MUTEX_CHECKS)
    mutex.implicitLock();
#endif /* DCDBG_USE_MUTEX_CHECKS */
    return true;
  }
  if (ret != 0) {
    cerr << "DC_Cond::wait() failed pthread_cond_wait - " << endl;
    // CLH - not MT safe - << strerror(errno) << endl;
  }
#if defined(DCDBG_USE_MUTEX_CHECKS)
  else {
    mutex.implicitLock();
  }
#endif /* DCDBG_USE_MUTEX_CHECKS */
  return false;
}


// ****************************************************************************
