// -*- c++ -*-
// ****************************************************************************
//  Name: dc_exec.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 05/19/1997 : mdb - Created.
//        12/10/2001 : mdb - Added ExecCollectOutput routines.
//        12/11/2001 : mdb - Removed explicit closestdin state and added stdin
//                           handling to ForkExec.
//  Desc: Routines for executing external commands (rsh, rexec, exec)
// ****************************************************************************
/** @file dc_exec.h */

#if !defined(H_EXEC)
#define H_EXEC

#ifndef _WIN32

#include "dc_standard.h"
#include "dc_argv.h"

#include <sys/types.h>

/// Class for executing external commands (rsh, rexec, exec)
class Exec {
public:
  //Exec(void) {};
  //~Exec(void);

  int Chdir(char *sbCwd);
  void Execvp(Argv &a);

  enum eFdType { FD_PIPE=-1, FD_PARENT=-2, FD_CLOSE=-3 };

  pid_t ForkExec(Argv &a, int &fdStdin, int &fdStdout, int &fdStderr,
		 char *sbCwd=NULL);
  pid_t ForkExec(Argv &a, char *sbCwd=NULL);
  pid_t ForkExec(Argv &a, char *sbStdout, char *sbStderr, char *sbCwd=NULL);
  int Wait(pid_t pid);

  enum eRshType { REXEC_REMSH, REXEC_RSH, REXEC_SSH };

  // mostly applies to REXEC_SSH
  enum eRshFlags {
    REXEC_FLAGS_ALLOCATETTY = 1,       // ssh -t
    REXEC_FLAGS_NOREADSTDIN = 2,       // ssh -n
    REXEC_FLAGS_NOFORWARDX11 = 4,      // ssh -x
    REXEC_FLAGS_AUTOADDHOSTKEYS = 8    // ssh -o "StrictHostKeyChecking=no"
  };

  void AppendRcmd(Argv &aNew,
		  int eType, char *sbHost, char *sbRemoteCmd,
		  char *sbUser=NULL, int eFlags=REXEC_FLAGS_ALLOCATETTY);

  char *ExecCollectOutput_Alloc(Argv &aExec);
  char *ExecCollectOutput_Alloc(char *sbExec);
};

#endif /* _WIN32 */
#endif /* H_EXEC */
// ****************************************************************************
