// ****************************************************************************
//  Name: dc_file.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 12/19/2001 : mdb - Created.
//  Desc: Supporting file i/o routines.
// ****************************************************************************

#include "dc_standard.h"
#include "dc_file.h"


// ctor
File::File(void) : sbFile(NULL), fd(-1), fClose(true) {
};

// ctor
File::File(char *sbFile_in) : sbFile(StrDup(sbFile_in)), fd(-1), fClose(true) {
};

// dtor
File::~File(void) {
  if (fClose) close();
  TestDelete(sbFile);
};

char *File::set(char *sbFile_in) {
  if (fd != -1) close();
  TestDelete(sbFile);
  return (sbFile = StrDup(sbFile_in));
}

// fClose_in: enables/disables auto close on dtor/open, etc.
void File::setFd(int fd_in, bool fClose_in) {
  if (fd_in < 0) return;
  fd = fd_in;
  fClose = fClose_in;
}

char *File::get(void) {
  return sbFile;
}

int File::open(int flags, int mode) {
  if (fClose) close();
  if (!sbFile) return -1;
  fd = ::open(sbFile, flags, mode);
  return fd;
}

int File::creat(int mode) {
  if (fClose) close();
  if (!sbFile) return -1;
  fd = ::creat(sbFile, mode);
  return fd;
}

bool File::isopen(void) {
  return (fd != -1 ? true : false);
}

int File::close(void) {
  if (fd == -1) return -1;  
  return ::close(fd);
}

int File::read(char *pBuf, int wSize) {
  if (fd == -1) return -1;
  return ::read(fd, pBuf, wSize);
}
int File::read(MemoryBuf &mb) {
  if (fd == -1) return -1;
  return mb.readMax(fd);
}

int File::write(char *pBuf, int wSize) {
  if (fd == -1) return -1;
  return ::write(fd, pBuf, wSize);
}

int File::write(MemoryBuf &mb) {
  if (fd == -1) return -1;
  return ::write(fd, mb.getPtr(), mb.getSize());
}

int File::lseek(off_t offset, int whence) {
  if (fd == -1) return -1;
  return ::lseek(fd, offset, whence);
}

int File::unlink(void) {
  if (fd != -1) ::close(fd);  // we must close it
  if (!sbFile) return -1;
  return ::unlink(sbFile);
}

int File::stat(struct stat *buf) {
  if (!sbFile) return -1;
  return ::stat(sbFile, buf);
}

#ifndef _WIN32
int File::lstat(struct stat *buf) {
  if (!sbFile) return -1;
  return ::lstat(sbFile, buf);
}
#endif

unsigned int File::checksum(void) {
  unsigned int wSum;
  if (!sbFile) return 0;
  int fdSave = fd;
  fd = ::open(sbFile, O_RDONLY);
  if (fd >= 0) {
    MemoryBuf mb(MB_1);
    wSum = mb.ChecksumBlockInit();
    while (read(mb) > 0) {
      wSum = mb.ChecksumBlock(wSum);
    }
    wSum = mb.ChecksumBlockDone(wSum);
    ::close(fd);
  }
  fd = fdSave;
  return wSum;
}



// ****************************************************************************
