// -*- c++ -*-
// ****************************************************************************
//  Name: dc_file.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 12/19/2001 : mdb - Created.
//  Desc: Supporting file i/o routines.
// ****************************************************************************
/** @file dc_file.h */

#if !defined(H_DC_FILE)
#define H_DC_FILE

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#ifndef _WIN32
#include <unistd.h>
#else
#include <io.h>
#endif

#include "dc_memory.h"

/// Class supporting file i/o routines.
class File {
protected:
  char *sbFile;
  int   fd;
  bool  fClose;
public:
  File(void);
  File(char *sbFile_in);
  ~File(void);
public:
  char *set(char *sbFile_in);
  void  setFd(int fd_in, bool fClose_in);
  char *get(void);

  #ifndef _WIN32
    int   open(int flags=O_RDONLY, int mode=S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
    int   creat(int mode=S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
  #else
    int   open(int flags=O_RDONLY, int mode=S_IREAD|S_IWRITE);
    int   creat(int mode=S_IREAD|S_IWRITE);  
  #endif

  bool  isopen(void);
  int   close(void);

  int   read(char *pBuf, int wSize);
  int   read(MemoryBuf &mb);
  int   write(char *pBuf, int wSize);
  int   write(MemoryBuf &mb);
  int   lseek(off_t offset, int whence);

  int   unlink(void);
  int   stat(struct stat *buf);
#ifndef _WIN32
  int   lstat(struct stat *buf);
#endif
  unsigned int checksum(void);
};


#endif /* H_DC_FILE */
// ****************************************************************************
