// -*- c++ -*-
// ****************************************************************************
//  Name: dc_guard.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 11/27/2000 : mdb - Created.
//        12/10/2000 : mdb - Fixed to work on an external mutex.
//        03/06/2001 : mdb - Added DC_GuardIf version.
//  Desc: Block level automatic mutex lock/unlock.
// ****************************************************************************
/** @file dc_guard.h */

#if !defined(H_DC_GUARD)
#define H_DC_GUARD

#include "dc_mutex.h"

//
//  DC_Guard
//
/**
 * Simple abstraction to take advantage of C++ object dtor to auto
 * unlock the given mutex when leaving the current scope (basic
 * block, fcn, whatever)
 */
class DC_Guard {
private:
  DC_Mutex *pmutex;
public:
  DC_Guard(DC_Mutex *pmutex_in) : pmutex(pmutex_in) {
    pmutex->lock();
  };
  DC_Guard(DC_Mutex &mutex_in) : pmutex(&mutex_in) {
    pmutex->lock();
  };
  ~DC_Guard(void) {
    pmutex->unlock();
  };
};

//
//  DC_GuardIf
//
/**
 * Adds a test predicate to the guard handling.  If predicate is true,
 * it operates as a regular guard, otherwise it does no locking.
 */
class DC_GuardIf {
private:
  DC_Mutex *pmutex;
public:
  DC_GuardIf(DC_Mutex *pmutex_in, bool fPredicate) : pmutex(pmutex_in) {
    if (fPredicate) pmutex->lock();
    else pmutex = NULL;
  };
  DC_GuardIf(DC_Mutex &mutex_in, bool fPredicate) : pmutex(&mutex_in) {
    if (fPredicate) pmutex->lock();
    else pmutex = NULL;
  };
  ~DC_GuardIf(void) {
    if (pmutex) pmutex->unlock();
  };  
};


#endif /* H_DC_GUARD */
// ****************************************************************************
