// ****************************************************************************
//  Name: dc_initcollect.h
//  Athr: Michael D. Beynon (mdb) - beynon@cs.umd.edu
//  Date: 09/05/2000 : mdb - Created.
//  Desc: Specialized synchronization structure between two threads.
// ****************************************************************************
/** @file dc_initcollect.h */

#if !defined(H_DC_INITCOLLECT)
#define H_DC_INITCOLLECT

#include "dc_semaphore.h"

/// Specialized synchronization structure between two threads.
/**
 * Allows for an initator thread to performs multiple operations that will
 * each cause a detector thread to later receive results.  The problem is
 * the results can start arriving before the initiator has completed its
 * operations that cause the results.
 */
// DC_InitiatorCollector_t ID(true);  // setup before
// 
// initiator thread(pID): {		collector thread(pID): {
//      :				    :
//      :				  while (1) {
//   // perform long ops		       :
//   // compute nResults expected	    // recv a result
//      :				    // (blocks for initiator)
//      :				    pID->collectorPart1(nRecv);
//   // blocks until all results recv	    // user processing of result
//   pID->initiator(nResults);	    	    pID->collectorPart2();
//   // continue			  }
//      :				    :
// }					}
//
class DC_InitiatorCollector_t {
private:
  bool fSync, fDone;
  int  nResults, cResults;
  DC_SemaphoreExtLock sem1, sem2;
  DC_Mutex mutex;
public:
  DC_InitiatorCollector_t(bool fSync_in=true);

  void reset(bool fSync_in=true);

  void initiator(int nResults_in);
  void initiatorPart1(int nResults_in);
  void initiatorPart2(void);

  void collector(int nRecv=1);
  void collectorPart1(int nRecv=1);
  void collectorPart2(void);
};

#endif /* H_DC_INITCOLLECT */
// ****************************************************************************
