// ****************************************************************************
//  Name: dc_library.cpp
//  Athr: Mike Gray (mgray)
//  Date: 05/20/2002 : mgray - Created.
//  Desc: Supporting dynamic library functionality
// ****************************************************************************

#include "dc_standard.h"
#include "dc_library.h"
#include <dlfcn.h>

// ctor
Library::Library(void) : filename(NULL), libHandle(NULL), loaded(false) {
};

// ctor
Library::Library(char *filename_in) : filename(StrDup(filename_in)), libHandle(NULL), loaded(false) {
};

// dtor
Library::~Library(void) {
  unload();
  TestDelete(filename);
};

char *Library::set(char *filename_in) {
  unload();
  TestDelete(filename);
  return (filename = StrDup(filename_in));
}

int Library::load(void)
{
  if (!loaded && NULL != filename) {
#ifdef WIN32
#error This code has never been tested.
    libHandle = (void *)LoadLibrary();
#else
    libHandle = dlopen(filename, RTLD_GLOBAL | RTLD_NOW);
#endif
    if (NULL != libHandle) {
      loaded = true;
      return 0;
    } else {
      return -1;
    }
  } else {
    return 0;
  }
}

void Library::unload(void)
{
  if (loaded) {
#ifdef WIN32
#error This code has never been tested.
    FreeLibrary(libHandle);
#else
    dlclose(libHandle);
#endif
    loaded = false;
    libHandle = NULL;
  }
}

const char *Library::getErrorMsg(void)
{
#ifdef WIN32
#error This code has never been tested.
  /* use FormatMessage(..., GetLastError(), ...); */
  /* http://msdn.microsoft.com/library/default.asp?url=/library/en-us/debug/errors_0sdh.asp */
#else
  return dlerror();
#endif
}

void *Library::getFunction(char *functionName)
{
  if (loaded) {
#ifdef WIN32
#error This code has never been tested.
    return GetProcAddress(libHandle, functionName);
#else
    return dlsym(libHandle, functionName);
#endif
  } else {
    return NULL;
  }
}

// ****************************************************************************

