// ****************************************************************************
//  Name: dc_lockedcounter.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 03/03/2001 : mdb - Created.
//  Desc: Simple synchronization primitive for counting.
// ****************************************************************************

#include "dc_lockedcounter.h"


//
//  DC_LockedCounter
//

// ctor
DC_LockedCounter::DC_LockedCounter(void)
  : num(0), rgCount(NULL) {
}

DC_LockedCounter::DC_LockedCounter(int num_in)
  : num(0), rgCount(NULL) {
  init(num_in);
}

// dtor
DC_LockedCounter::~DC_LockedCounter(void) {
  if (rgCount) delete [] rgCount;
}

// unlocked
void DC_LockedCounter::init(int num_in) {
  if (rgCount) delete [] rgCount;

  num = num_in;
  if (num > 0) {
    rgCount = new int[num];
    for (int i=0; i<num; ++i) {
      rgCount[i] = 0;
    }
  } else {
    rgCount = NULL;
  }
}

int DC_LockedCounter::set_u(int i, int wValue) {
  return (rgCount[i] = wValue);
}

int DC_LockedCounter::set(int i, int wValue) {
  int wRtn;
  mutex.lock();
  wRtn = (rgCount[i] = wValue);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounter::inc_u(int i, int w) {
  return (rgCount[i] += w);
}

int DC_LockedCounter::inc(int i, int w) {
  int wRtn;
  mutex.lock();
  wRtn = (rgCount[i] += w);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounter::dec_u(int i, int w) {
  return (rgCount[i] -= w);
}

int DC_LockedCounter::dec(int i, int w) {
  int wRtn;
  mutex.lock();
  wRtn = (rgCount[i] -= w);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounter::get_u(int i) {
  return rgCount[i];
}

int DC_LockedCounter::get(int i) {
  int wRtn;
  mutex.lock();
  wRtn = rgCount[i];
  mutex.unlock();
  return wRtn;
}


//
//  DC_LockedCounterThreshold
//

// ctor
DC_LockedCounterThreshold::DC_LockedCounterThreshold(void)
  : rgLow(NULL), rgHigh(NULL) {
}

DC_LockedCounterThreshold::DC_LockedCounterThreshold(int num_in,
						     int wLow, int wHigh)
  : rgLow(NULL), rgHigh(NULL) {
  init(num_in, wLow, wHigh);
}

// dtor
DC_LockedCounterThreshold::~DC_LockedCounterThreshold(void) {
  if (rgLow) delete [] rgLow;
  if (rgHigh) delete [] rgHigh;
}

// unlocked
void DC_LockedCounterThreshold::init(int num_in, int wLow, int wHigh) {
  DC_LockedCounter::init(num_in);

  if (rgLow) delete [] rgLow;
  if (rgHigh) delete [] rgHigh;

  if (num > 0) {
    rgLow = new int[num];
    rgHigh = new int[num];
    for (int i=0; i<num; ++i) {
      rgLow[i] = wLow;
      rgHigh[i] = wHigh;
    }
  } else {
    rgLow = rgHigh = NULL;
  }
}

int DC_LockedCounterThreshold::setLow(int i, int wValue) {
  int wRtn;
  mutex.lock();
  wRtn = setLow_u(i, wValue);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounterThreshold::getLow(int i) {
  int wRtn;
  mutex.lock();
  wRtn = getLow_u(i);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounterThreshold::setHigh(int i, int wValue) {
  int wRtn;
  mutex.lock();
  wRtn = setHigh_u(i, wValue);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounterThreshold::getHigh(int i) {
  int wRtn;
  mutex.lock();
  wRtn = getHigh_u(i);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounterThreshold::incWaitHigh_u(int i, int w) {
  while (get_u(i) >= getHigh_u(i)) {
    cvNotHigh.wait(mutex);
  }
  return inc_u(i);
}

int DC_LockedCounterThreshold::incWaitHigh(int i, int w) {
  int wRtn;
  mutex.lock();
  wRtn = incWaitHigh_u(i, w);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounterThreshold::decSignalHigh_u(int i, int w) {
  int wRtn = dec_u(i);
  if (wRtn == getHigh_u(i) - 1) {
    cvNotHigh.signal();
  }
  return wRtn;
}

int DC_LockedCounterThreshold::decSignalHigh(int i, int w) {
  int wRtn;
  mutex.lock();
  wRtn = decSignalHigh_u(i, w);
  mutex.unlock();
  return wRtn;
}

int DC_LockedCounterThreshold::diffHigh(int i) {
  int wRtn;
  mutex.lock();
  wRtn = diffHigh_u(i);
  mutex.unlock();
  return wRtn;
}


// ****************************************************************************
