// -*- c++ -*-
// ****************************************************************************
//  Name: dc_lockedcounter.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 03/03/2001 : mdb - Created.
//        06/11/2001 : mdb - Added high and low thresholds and signal/block.
//  Desc: Simple synchronization primitive for counting.
// ****************************************************************************
/** @file dc_lockedcounter.h */

#if !defined(H_DC_LOCKEDCOUNTER)
#define H_DC_LOCKEDCOUNTER

#include "../common-config.h"

#include "dc_mutex.h"
#include "dc_cond.h"

/// Simple synchronization primitive for counting.
class DC_LockedCounter {
protected:
  DC_Mutex mutex;
  int  num;
  int *rgCount;

public:
  DC_LockedCounter(void);
  DC_LockedCounter(int num_in);
  ~DC_LockedCounter(void);

  void init(int num_in);

  void lockInternal(void) { mutex.lock(); };
  void unlockInternal(void) { mutex.unlock(); };

  int set_u(int i, int wValue);
  int set(int i, int wValue);
  int inc_u(int i, int w=1);
  int inc(int i, int w=1);
  int dec_u(int i, int w=1);
  int dec(int i, int w=1);
  int get_u(int i);
  int get(int i);
};

/// Simple synchronization primitive for counting.
class DC_LockedCounterThreshold : public DC_LockedCounter {
protected:
  DC_Cond cvNotHigh, cvNotLow;
  int *rgLow;
  int *rgHigh;

public:
  DC_LockedCounterThreshold(void);
  DC_LockedCounterThreshold(int num_in, int wLow, int wHigh);
  ~DC_LockedCounterThreshold(void);

  void init(int num_in, int wLow, int wHigh);

  int setLow_u(int i, int wValue) { return (rgLow[i] = wValue); };
  int setLow(int i, int wValue);
  int getLow_u(int i) { return rgLow[i]; };
  int getLow(int i);
  int setHigh_u(int i, int wValue) { return (rgHigh[i] = wValue); };
  int setHigh(int i, int wValue);
  int getHigh_u(int i) { return rgHigh[i]; };
  int getHigh(int i);

  // will block if the high threshold state is reached
  int incWaitHigh_u(int i, int w=1);
  int incWaitHigh(int i, int w=1);
  // will signal if the high threshold state is exited
  int decSignalHigh_u(int i, int w=1);
  int decSignalHigh(int i, int w=1);

  void waitHigh(void) { cvNotHigh.wait(mutex); };

  // high threshold minus current value
  int diffHigh_u(int i) { return rgHigh[i] - get_u(i); };
  int diffHigh(int i);
  
};


#endif /* H_DC_LOCKEDCOUNTER */
// ****************************************************************************
