// ****************************************************************************
//  Name: dc_machine.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 02/24/1998 : mdb - Created.
//  Desc: Machine specific support routines.
// ****************************************************************************

//
//  Include Files
//
#include "dc_machine.h"


// ****************************************************************************
//  Name: IsBigEndian
//  Date: 02/24/1998 : mdb - description
// ****************************************************************************
/**
 * Detects the "endian-ness" of a machine.
 * @return 1=big endian, 0=little endian
 */
int IsBigEndian(void) {
  unsigned int w = 1;
  return (((char*)&w)[0] == 0);
}


// ****************************************************************************
//  Name: ConvertEndian
//  Date: 02/24/1998 : mdb - Copied from Renato's routine
// ****************************************************************************
/**
 * Converts an array of integers into the opposite endian format.
 * @param nElem - number of elements in array
 * @param sElem - size (in bytes) of each element
 * @param buf - ptr to start of array
 */
void ConvertEndian(int nElem, int sElem, void *buf) {
  unsigned char *pBuf = (unsigned char *)buf;
  unsigned char tmp;

  for (int i=0; i<nElem; i++) {
    for (int j=0; j<sElem/2; j++) {
      tmp = pBuf[i*sElem + j];
      pBuf[i*sElem + j] = pBuf[i*sElem + sElem - j - 1];
      pBuf[i*sElem + sElem - j - 1] = tmp;
    }
  }
}

void ConvertEndian(int nElem, int sElem, void *pSrc, void *pDst) {
  unsigned char *pBufSrc = (unsigned char *)pSrc;
  unsigned char *pBufDst = (unsigned char *)pDst;

  for (int i=0; i<nElem; i++) {
    for (int j=0; j<sElem; j++) {
      pBufDst[j] = pBufSrc[sElem - j - 1];
    }
    pBufSrc += sElem;
    pBufDst += sElem;
  }
}


// ****************************************************************************
