// ****************************************************************************
//  Name: dc_metric.cpp
//  Athr: Michael D. Beynon (mdb) - beynon@cs.umd.edu
//  Date: 06/04/1997 : mdb - Created.
//  Desc: Simple metrics.
// ****************************************************************************

#include "dc_metric.h"
#include "dc_missing.h"


// ****************************************************************************
//  Class: Metric
// ****************************************************************************


// ****************************************************************************
//  Name: NewValue
//  Date: 06/10/1997 : mdb - Created.
// ****************************************************************************
/**
 * Considers a new value for stats purposes.
 */
double Metric::NewValue(double dNew) {
  if (wNum == 0) {
    dMax = dMin = dSum = dNew;
    dSum2 = dNew * dNew;
  } else {
    if (dNew > dMax) dMax = dNew;
    if (dNew < dMin) dMin = dNew;
    dSum += dNew;
    dSum2 += dNew * dNew;
  }
  wNum++;
  return dNew;
}


// ****************************************************************************
//  Name: getVar
//  Date: 06/12/1997 : mdb - Created.
// ****************************************************************************
/**
 * Calculates the sample variance.
 */
double Metric::getVar(void) {
  double dMean = getMean();
  return (wNum < 2 ? double(0) :
	  (dSum2 - 2*dMean*dSum + double(wNum)*dMean*dMean)/(double(wNum-1)));
}


// ****************************************************************************
//  Name: operator<<
//  Date: 06/10/1997 : mdb - Created.
// ****************************************************************************
/**
 * Outputs the current metric value, and stats.
 */
ostream &operator<<(ostream &os, Metric &m) {
  if (m.wNum != 0) {
    return os << "n= " << m.wNum
	      << " Min= " << m.dMin
	      << " Max= " << m.dMax
	      << " Sum= " << m.dSum
	      << " Mean= " << m.getMean()
	      << " Var= " << m.getVar()
	      << " StdDev= " << m.getStdDev();
    return os; 
  } else {
    return os << "(uninitialized metric)";
  }
}


// ****************************************************************************
//  Class: MetricSamples
// ****************************************************************************


// ****************************************************************************
//  Name: operator<<
//  Date: 03/19/2000 : mdb - Created.
// ****************************************************************************
/**
 * Outputs the current metric value, and stats.
 */
ostream &operator<<(ostream &os, MetricSamples &m) {
  os << *(static_cast(Metric*,&m));
  for (Types::uint i=0; i<m.daSamples.NumElem(); ++i) {
    os << " [" << i << "]= " << m.daSamples[i];
  }
  return os;
}


// ****************************************************************************
