// ****************************************************************************
//  Name: dc_metric.h
//  Athr: Michael D. Beynon (mdb) - beynon@cs.umd.edu
//  Date: 06/10/1997 : mdb - Created.
//        06/12/1997 : mdb - Added sample variance and stddev.
//        01/09/2000 : mdb - Added getNum() missing fcn.
//        03/19/2000 : mdb - Subclass to save all samples. 
//  Desc: Simple metrics.  Should be made a template class for data type.
// ****************************************************************************
/** @file dc_metric.h */

#if !defined(H_METRIC)
#define H_METRIC

#include "dc_standard.h"
#include "dc_darray.h"

/// Simple metrics.  Should be made a template class for data type.
class Metric {
protected:
  long   wNum;
  double dMin, dMax, dSum, dSum2;

public:
  Metric(void) { Reset(); };

  double NewValue(double dNew);
  void Reset(void) { wNum = 0; dMin=dMax=dSum=dSum2=double(0.0); };

  long getNum(void) { return wNum; };
  double getMin(void) { return (wNum == 0 ? 0 : dMin); };
  double getMax(void) { return (wNum == 0 ? 0 : dMax); };
  double getSum(void) { return (wNum == 0 ? 0 : dSum); };
  double getMean(void) { return (wNum == 0 ? 0 : dSum/double(wNum)); };
  double getVar(void);
  double getStdDev(void) { return sqrt(getVar()); };

  friend ostream &operator<<(ostream &os, Metric &m);
};

/// Simple metrics.
class MetricSamples : public Metric {
protected:
  DArray<double> daSamples;

public:
  double NewValue(double dNew) {
    daSamples.Append(dNew);
    return Metric::NewValue(dNew);
  };

  friend ostream &operator<<(ostream &os, MetricSamples &m);
};


#endif /* H_METRIC */
// ****************************************************************************
