// ****************************************************************************
//  Name: dc_namevalue.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 03/11/1998 : mdb - Created.
//  Desc: Simple name/value pair storage.
// ****************************************************************************

#include "dc_namevalue.h"

/* #define DCDBG_NAMEVALUE */


// ****************************************************************************
//  Name: Add
//  Date: 03/11/1998 : mdb - Created.
// ****************************************************************************
/** 
 * Adds a new entry.
 * @param sbName - pair to add
 * @param sbEntry - pair to add
 * @return int - 0=added ok, !0=duplicate found
 */
int NameValue::Add(char *sbName, char *sbValue) {
  Types::uint i;
  NameValue_Entry_t e;

  if (Index(sbName, i) != 0) {
    e.sbName = StrDup(sbName);
    e.sbValue = StrDup(sbValue);
    rgEntry.Append(e);
    fFlush = true;
    return 0;
  }
  return 1;  // duplicate
}


// ****************************************************************************
//  Name: AddReplace
//  Date: 03/11/1998 : mdb - Created.
// ****************************************************************************
/**
 * Adds a new entry.
 * @param sbName - pair to add
 * @param sbEntry - pair to add
 * @return int - 0=added ok, !0=replaced existing value
 */
int NameValue::AddReplace(char *sbName, char *sbValue) {
  Types::uint i;
  NameValue_Entry_t e;

  if (Index(sbName, i) != 0) {
    e.sbName = StrDup(sbName);
    e.sbValue = StrDup(sbValue);
    rgEntry.Append(e);
    fFlush = true;
    return 0;
  }
  // replace
  TestDelete(rgEntry.Get(i).sbValue);
  rgEntry.Get(i).sbValue = StrDup(sbValue);
  return 1;  // duplicate
}


// ****************************************************************************
//  Name: Index
//  Date: 03/11/1998 : mdb - Created.
// ****************************************************************************
/**
 * Attempts to find an entry.
 * @param sbName - name to look for
 * @param i - index of name
 * @return int - 0=found, !0=not found
 */
int NameValue::Index(char *sbName, Types::uint &i) {
  for (i=0; i<rgEntry.NumElem(); i++) {
    if (StrCmp(sbName, rgEntry.Get(i).sbName, true) == 0) {
      return 0;  // found
    }
  }
  return 1;
}


// ****************************************************************************
//  Name: Find
//  Date: 03/11/1998 : mdb - Created.
// ****************************************************************************
/**
 * Attempts to find an entry.
 * @param sbName - name to look for
 * @return char * - NULL=not found, else=entry found
 */
char *NameValue::Find(char *sbName) {
  Types::uint i;
  if (Index(sbName, i) == 0) {
    return rgEntry.Get(i).sbValue;  // found
  }
  return NULL;
}


// ****************************************************************************
//  Name: Flush
//  Date: 03/11/1998 : mdb - Created.
// ****************************************************************************
/** 
 * Free up alloced space for strings in darrays.
 */
void NameValue::Flush(void) {
  Types::uint i;

  for (i=0; i<rgEntry.NumElem(); i++) {
    TestDeleteNull(rgEntry.Get(i).sbName);
    TestDeleteNull(rgEntry.Get(i).sbValue);
  }
  rgEntry.Flush();
}


// ****************************************************************************
//  Name: First
//  Date: 03/11/1998 : mdb - Created.
// ****************************************************************************
/**
 * Point iteration to first name.
 */
char *NameValue::FirstName(void) {
  iEntry = -1;
  return NextName();
}

// ****************************************************************************
//  Name: NextName
//  Date: 03/11/1998 : mdb - Created.
// ****************************************************************************
/**
 * Point iteration to the next name.
 */
char *NameValue::NextName(void) {
  if (iEntry+1 < (int)rgEntry.NumElem()) {
    iEntry++;
    return rgEntry.Get(iEntry).sbName;
  }
  return NULL;
}

// ****************************************************************************
//  Name: CurrentValue
//  Date: 03/11/1998 : mdb - Created.
// ****************************************************************************
/**
 * Get value from the current section and name.
 */
char *NameValue::CurrentValue(void) {
  if (iEntry < (int)rgEntry.NumElem()) {
    return rgEntry.Get(iEntry).sbValue;
  }
  return NULL;
}


ostream &operator<<(ostream &os, NameValue &nv) {
  char *sbName;

  os << "NameValue (" << nv.Num() << " entries)\n";

  sbName = nv.FirstName();
  while (sbName != NULL) {
    os << "  " << sbName << " = " << nv.CurrentValue() << endl;
    sbName = nv.NextName();
  }
  return os;
}

// ****************************************************************************
