// ****************************************************************************
//  Name: dc_pthread-wrappers.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 04/23/2001 : mdb - Moved into seperate file.
//  Desc: Pthreads wrappers and _np fcns.
// ****************************************************************************

#include "dc_pthread-wrappers.h"

#ifdef _WIN32
#include "unix_calls.h"
#include <pthread.h>
#include <windows.h>
#include <wtypes.h>
#define timezone _timezone
#else
#include <sys/time.h>
#include <sys/resource.h>
#endif

#include <errno.h>


#if defined(USEC_PER_SEC)
#undef USEC_PER_SEC
#endif
#define USEC_PER_SEC  1000000

#if defined(NSEC_PER_SEC)
#undef NSEC_PER_SEC
#endif
#define NSEC_PER_SEC  1000000000

#if defined(NSEC_PER_USEC)
#undef NSEC_PER_USEC
#endif
#define NSEC_PER_USEC 1000


#if !defined(ALPHA) && !defined(AIX)
// this is a _np (non-portable) routine that does not exist in solaris/linux
int pthread_get_expiration_np(const struct timespec *delta,
			      struct timespec *abstime) {
#ifndef _WIN32

  if (delta->tv_sec < 0 ||
      delta->tv_nsec < 0 || delta->tv_nsec >= NSEC_PER_SEC) {
    return EINVAL;
  }
#if defined(SUNOS5)
  clock_gettime(CLOCK_REALTIME, abstime);
#else
  struct timeval tv;
  struct timezone tz;
  if (gettimeofday(&tv, &tz) != 0) return 0; // failed
  abstime->tv_sec = tv.tv_sec;
  abstime->tv_nsec = tv.tv_usec / 1000;
#endif
  abstime->tv_sec += delta->tv_sec;
  abstime->tv_nsec += delta->tv_nsec;
  if (abstime->tv_nsec >= NSEC_PER_SEC) {
    int sec = abstime->tv_nsec / NSEC_PER_SEC;
    abstime->tv_sec += sec;
    abstime->tv_nsec -= sec * NSEC_PER_SEC;
  }
#endif

  return 0;
}
#endif


// ****************************************************************************
