// -*- c++ -*-
// ****************************************************************************
//  Name: dc_pthread-wrappers.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 04/23/2001 : mdb - Moved into seperate file.
//  Desc: Pthreads wrappers and _np fcns.
// ****************************************************************************
/** @file dc_pthread-wrappers.h */

#if !defined(H_PTHREAD_WRAPPERS)
#define H_PTHREAD_WRAPPERS

#include "../common-config.h"

#include <pthread.h>


// ****************************************************************************
//  Thread fcn wrappers
//  - use current standard calls, and xlate to D4 versions (DCE)
// ****************************************************************************
inline int Pthread_mutex_init(pthread_mutex_t *pmutex,
			      pthread_mutexattr_t *pattr) {
#if defined(CONFIG_HAVE_PTHREAD_D4)
  // must use constant instead of NULL
  return pthread_mutex_init(pmutex, pattr ? pattr : pthread_mutexattr_default);
#else /* CONFIG_HAVE_PTHREAD_D4 */
  return pthread_mutex_init(pmutex, pattr);
#endif /* CONFIG_HAVE_PTHREAD_D4 */
};

inline int Pthread_cond_init(pthread_cond_t *pcond,
			     pthread_condattr_t *pattr) {
#if defined(CONFIG_HAVE_PTHREAD_D4)
  // must use constant instead of NULL
  return pthread_cond_init(pcond, pattr ? pattr : pthread_condattr_default);
#else /* CONFIG_HAVE_PTHREAD_D4 */
  return pthread_cond_init(pcond, pattr);
#endif /* CONFIG_HAVE_PTHREAD_D4 */
};

inline int Pthread_create(pthread_t *pthread,
			  pthread_attr_t *pattr,
			  void *(*pstart_fcn)(void *),
			  void *parg) {
#if defined(CONFIG_HAVE_PTHREAD_D4)
  // must use attr or constant instead of addr of attr
  return pthread_create(pthread, pattr ? *pattr : pthread_attr_default,
			pstart_fcn, parg);
#else /* CONFIG_HAVE_PTHREAD_D4 */
  return pthread_create(pthread, pattr, pstart_fcn, parg);
#endif /* CONFIG_HAVE_PTHREAD_D4 */
};

inline int Pthread_detach(pthread_t &thread) {
#if defined(CONFIG_HAVE_PTHREAD_D4)
  // must use addr of thread
  return pthread_detach(&thread);
#else /* CONFIG_HAVE_PTHREAD_D4 */
  return pthread_detach(thread);
#endif /* CONFIG_HAVE_PTHREAD_D4 */
};


#if !defined(ALPHA) && !defined(AIX)
extern int pthread_get_expiration_np(const struct timespec *delta,
				     struct timespec *abstime);
#endif

#endif /* H_PTHREAD_WRAPPERS */
// ****************************************************************************
