// -*- c++ -*-
// ****************************************************************************
//  Name: dc_random.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 01/28/1998 : mdb - Created from example in the [Jain] book, and
//                           my previous version.
//  Desc: Pseudo-random multiplicitive linear congruential generator (LCG).
//          X[n] = 7^5 * X[n-1] mod (2^31 - 1)
// ****************************************************************************
/** @file dc_random.h */

#ifndef H_RANDOM
#define H_RANDOM


//
//  System Include Files
//
#include "dc_standard.h"


//
//  Class Definitions
//

/// Pseudo-random multiplicitive linear congruential generator (LCG).
class Random {
  int wSeed;  // random value seed

  int  wCurrentRandomValue;  // value we are using for 
  int  cBitsLeft;  // unused bits left in wCurrentRandomValue

  bool fConserveBits;

  int a;
  int m;
  int q;
  int r;

  void _Init(void) {
    a = 16807;      // 7^5 multiplier
    m = 2147483647; // 2^31 - 1 modulus
    q = 127773;     // m div a
    r = 2836;       // m mod a
  };

public:
  // ctor
  Random(void);
  Random(int wSeed_in);
  //~Random(void);

  void SetSeed(int wSeed_in) { wSeed = wSeed_in; };
  void SetConserveBits(bool f) { fConserveBits = f; };

  int GetSeed(int i);  // get seed[0..49] that are 100k apart

  int GetLastRand(void) { return wSeed; };
  int GetRand(void);  // return the next random 31 bit integer
  double GetRand01(void) { return (double) GetRand() / (double) m; }; // [0..1)
  int GetRandRange(int wLow, int wHigh);

  // note this is always bit conserving, because I haven't changed it yet!
  bool ProbTest(int Qn, int Qd);  // handle a Qn/Qd coin toss
};


#endif // H_RANDOM
/*****************************************************************************/
