// ****************************************************************************
//  Name: dc_semaphore.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 08/12/2000 : mdb - Created.
//        03/06/2001 : mdb - Added wait_testblock().
//  Desc: Simple counting semaphore.
// ****************************************************************************

#include "dc_semaphore.h"


//#define DCDBG_SEMAPHORE

#if defined(DCDBG_SEMAPHORE)
#ifdef _WIN32
#include <iostream>
#else
#include <iostream.h>
#endif
#endif /* DCDBG_SEMAPHORE */


//
//  class DC_Semaphore
//

// ctor/dtor
DC_Semaphore::DC_Semaphore(int n_in) : n(n_in) {
}

/**
 * do not call this if any threads are blocked
 */
void DC_Semaphore::init(int n_in) {
  n = n_in;
}

/**
 * @return T=wait() would have blocked
 */
bool DC_Semaphore::wait_testblock(void) {
  bool fRtn;
  mutex.lock();
  fRtn = (n-1 < 0) ? true : false;
  mutex.unlock();
  return fRtn;
}

/**
 * synonyms: P, down, acquire
 */
void DC_Semaphore::wait(void) {
  mutex.lock();
  --n;
  while (n < 0) {
    cv.wait(mutex);
  }
  mutex.unlock();
}

/**
 * synonyms: V, up, release
 */
void DC_Semaphore::post(void) {
  mutex.lock();
  ++n;
  if (n <= 0) {  // there are waiters blocked
    cv.signal();
  }
  mutex.unlock();
}


//
//  class DC_SemaphoreExtLock
//

// ctor
DC_SemaphoreExtLock::DC_SemaphoreExtLock(int n_in) : n(n_in) {
}

void DC_SemaphoreExtLock::setExtLock(DC_Mutex *pmutex_in) {
  pmutex = pmutex_in;
}

/**
 * do not call this if any threads are blocked
 */
void DC_SemaphoreExtLock::init(int n_in) {
  n = n_in;
}

/**
 * @return T=wait() would have blocked
 */
bool DC_SemaphoreExtLock::wait_testblock(void) {
  return ((n-1 < 0) ? true : false);
}

/**
 * synonyms: P, down, acquire
 */
void DC_SemaphoreExtLock::wait(void) {
  --n;
  while (n < 0) {
    cv.wait(*pmutex);
  }
}

/**
 * synonyms: V, up, release
 */
void DC_SemaphoreExtLock::post(void) {
  ++n;
  if (n <= 0) {  // there are waiters blocked
    cv.signal();
  }
}


// ****************************************************************************
