// -*- c++ -*-
// ****************************************************************************
//  Name: dc_semaphore.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 08/12/2000 : mdb - Created.
//        03/06/2001 : mdb - Added wait_testblock().
//  Desc: Simple counting semaphore.
// ****************************************************************************
/** @file dc_semaphore.h */

#if !defined(H_DC_SEMAPHORE)
#define H_DC_SEMAPHORE

#include "dc_mutex.h"
#include "dc_cond.h"

/// Simple counting semaphore.
/**
 * initial value of n represents the number of wait() operations
 * are pre-loaded to succeed without any post() calls.  more posts
 * add to this value, waits subtract.  A wait will block when the
 * value is <= 0.
 *
 * usage:
 * - 2 process barrier: ctor(0), t1 wait() will block until t2 post().
 */
class DC_Semaphore {
private:
  int n;

  DC_Mutex mutex;
  DC_Cond  cv;

public:
  DC_Semaphore(int n_in=0);

  // do not call this if any threads are blocked
  void init(int n_in);  

  bool wait_testblock(void);  // return: T=wait() would have blocked
  void wait(void);  // synonyms: P, down, acquire  
  void post(void);  // synonyms: V, up, release
};

/**
 * the unlocked version requires an external locking mechanism, and assumes
 * the lock is held for all calls, and uses it for the internal condvar.
 */
class DC_SemaphoreExtLock {
private:
  int n;

  DC_Mutex *pmutex;
  DC_Cond   cv;

public:
  DC_SemaphoreExtLock(int n_in=0);

  void setExtLock(DC_Mutex *pmutex_in);

  // do not call this if any threads are blocked
  void init(int n_in);

  bool wait_testblock(void);  // return: T=wait() would have blocked
  void wait(void);  // synonyms: P, down, acquire
  void post(void);  // synonyms: V, up, release
};


#endif /* H_DC_SEMAPHORE */
// ****************************************************************************
