// -*- c++ -*-
// ****************************************************************************
//  Name: dc_standard.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 06/17/1995 : mdb - Created.
//  Desc: 
// ****************************************************************************
/** @file dc_standard.h */

#if !defined(H_STANDARD)
#define H_STANDARD

//
//  System Include Files
//
#if defined(CONFIG_NEED_CDEFS_H)
#  include <sys/cdefs.h>   // this fixes a problem with redefining __P()
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <errno.h>

#include <iostream>
using namespace std;


#include "../common-config.h"


#include "dc_types.h"

//
//  Declarations
//
extern const int wWARNING;
extern const int wERROR;
extern const int wFATAL;

extern char *sbProgramName;
extern char *sbProgramDir;


//
//  Function prototypes
//
extern int StandardInit(int argc, char *argv[]);
extern int StandardCleanup(void);
extern int StrCmp(const char *sbStr1, const char *sbStr2,
		  bool fCaseSensitive=false);
extern int StrNCmp(const char *sbStr1, const char *sbStr2, int n,
		   bool fCaseSensitive=false);
extern char *StrDup(const char *sb);
extern char *StrNDup(const char *sb, int n);
extern void  StrNCopyNull(char *sbDest, const char *sbSrc, int cchMax);
extern char *StrToLower(char *sb);
extern char *StrToUpper(char *sb);
extern char *StrLeftTrimWhite(char *sb);
extern char *StrRightTrimWhite(char *sb);
extern char *StrTrimWhite(char *sb);
extern char *MemNDup(char *p, int n);
extern char *strrstr(const char *sbStr, const char *sbSub);
extern char *StrEndDelim(char *sbStr, char *sbEnd, char *sbStart=NULL);

extern char *ErrorHeader(int wSeverity);
extern char *ErrorIndent(void);
extern char NextChar(istream &is);
extern istream &EatWhite(istream &is);
extern istream &EatLine(istream &is);

extern int NumberDecimalDigits(long x);
extern int NumberBits(long x);
extern double lg(double x);
extern bool IsPrime(long wN);

extern long Atoi(const char *sb, long wDefault=0);
extern double Atof(const char *sb, double rDefault=0.0);
extern char *CalcBytes(long wBytes);
extern char *CalcBytes_r(long wBytes, char *sbTmp);  // MT-safe
extern char *CalcRate(long wBytes, double dTime);
extern char *CalcRate_r(long wBytes, double dTime, char *sbTmp);  // MT-safe
extern int CalcPercent(int wNum, int wTotal);


//
//  Macros / Inlines
//
#define StandardInRange(low, x, high) ((((x) >= (low)) && ((x) <= (high))) ? true : false)
#define StandardMax(x, y) (((x) > (y)) ? (x) : (y))
#define StandardMin(x, y) (((x) > (y)) ? (y) : (x))

#define DeleteNull(v) { delete [] v; v = NULL; }
#define TestDelete(v) if (v) { delete [] v; }
#define TestDeleteNull(v) if (v) { delete [] v; v = NULL; }

#define SDeleteNull(v) { delete v; v = NULL; }
#define STestDelete(v) if (v) { delete v; }
#define STestDeleteNull(v) if (v) { delete v; v = NULL; }


#define KB_1          1024
#define KB_2          2048
#define KB_4          4096
#define KB_8          8192
#define KB_16        16384
#define KB_32        32768
#define KB_64        65536
#define KB_128      131072
#define KB_256      262144
#define KB_512      524288
#define MB_1       1048576
#define MB_2       2097152
#define MB_4       4194304
#define MB_8       8388608
#define MB_16     16777216 
#define MB_32     33554432
#define MB_64     67108864
#define MB_128   134217728
#define MB_256   268435456
#define MB_512   536870912
#define GB_1    1073741824

#define K_1           1000
#define M_1        1000000
#define G_1     1000000000


#endif /* H_STANDARD */
// ****************************************************************************
