// -*- c++ -*-
// ****************************************************************************
//  Name: dc_str.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 07/28/1997 : mdb - Created.
//        04/27/1998 : mdb - Added NCat().
//  Desc: Supporting string routines.
// ****************************************************************************
/** @file dc_str.h */

#if !defined(H_STR)
#define H_STR

#include "dc_standard.h"
#include "dc_darray.h"

/// Supporting string class
class Str {
  char *sb;
  int   wLen;  // length of null terminated string
  int   wMax;  // size of allocated space

  char *sbTmp; // used for Token()
  
public:
  Str(void);
  Str(const char *sb);
  ~Str(void);

  char *Set(const char *sbNew);
  char *SetNoDup(char *sbNew);
  char *Get(void) { return sb; };
  int Len(void) { return wLen; };

  char *Steal(void);

  char *Dup(void) { return StrDup(sb); };
  char *Cat(char chAdd);
  char *Cat(char *sbAdd);
  char *NCat(char *sbAdd, int wLenAdd);
  char *Replace(char chOld, char chNew);
  char *Replace(char *sbOld, char *sbNew);
  char *ReplaceEnvVars(void);
  char *RChop(char *sbDelim);
  char *RChopCh(char chChop);

private:
  char *_Token_Internal(char *sbDelim, int wMaxSize, bool fNoDup, bool fCount);
public:
  void TokenReset(void) { sbTmp = NULL; };
  int TokenNum(char *sbDelim = " \t\n", int wMaxSize = -1);
  char *Token(char *sbDelim = " \t\n", int wMaxSize = -1) {
    return _Token_Internal(sbDelim, wMaxSize, false, false);
  };
  char *TokenNoDup(char *sbDelim = " \t\n", int wMaxSize = -1) {
    return _Token_Internal(sbDelim, wMaxSize, true, false);
  };
  DArray<char*> *TokenArray(char *sbDelim = " \t\n", int wMaxSize = -1);
  static void FlushTokenArray(DArray<char*> *pda);
  DArray<char*> *TokenArrayNoDup(char *sbDelim = " \t\n", int wMaxSize = -1);

  char *TokenGet(int i, char *sbDelim = " \t\n", int wMaxSize = -1);

  friend ostream &operator<<(ostream &os, Str &str);
};


#endif /* H_STR */
// ****************************************************************************
