// -*- c++ -*-
// ****************************************************************************
//  Name: dc_timer.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 06/04/1997 : mdb - Created.
//        04/06/2000 : mdb - Semantics change to allow timer to be stopped and
//                           then restarted without losing the elapsed time. A
//                           Reset is used to clear the running elapsed time.
//  Desc: Simple timers.
// ****************************************************************************
/** @file dc_timer.h */

#if !defined(H_TIMER)
#define H_TIMER

#ifdef _WIN32
#include <time.h>
#include <winsock.h>
#else
#include <sys/time.h>
#include <sys/resource.h>
#endif

#include <iostream>
using namespace std;


#include "dc_standard.h"

extern double w_seconds(void);  // defined in wclk.cpp
extern ostream &operator<<(ostream &os, struct timeval &tv);

/// Simple timer.
class Timer {
protected:
  struct timeval tvElapsed;  // time elapsed so far between start/stop calls
  struct timeval tvStart;    // time of last start
  enum   eState_t { INIT, STARTED, STOPPED } eState;

  void _Correction(struct timeval &tv);

public:
  Timer(void) { Reset(); };

  void Reset(void);  // Clears any elapsed time, and start time.
  void Start(void);  // Starts the timer
  void Stop(void);   // Stops the timer, and calculates the elapsed time.

  double Elapsed(void); // If running, stops the timer, and calc elapsed time.
  double Snapshot(void); // Calc elapsed time w/o stopping the timer.

  friend ostream &operator<<(ostream &os, Timer &t);
};

/// Simple timer.
class CpuDelay {
  Timer t;
  double rDelayFactor;
  double rMaxDelay;
  bool fNoDelay;
public:
  CpuDelay(int wDelayPercent, double rMaxDelay_in = 0.0);

  void Begin(void);  // start region
  double End(bool fDebug=false);  // end region - busywait for remaining time

  // utility fcns
  void BusyWait(double rSeconds);
  void Sleep(double rSeconds);
};


#endif /* H_TIMER */
// ****************************************************************************
