// -*- c++ -*-
// ****************************************************************************
//  Name: thread-reader.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 04/08/2000 : mdb - Created.
//  Desc: Reader execution thread and supporting functions.
// ****************************************************************************

#if !defined(H_DC_THREAD_READER)
#define H_DC_THREAD_READER

#include "lib/dc_threadclass.h"
#include "lib/dc_barrier.h"
#include "lib/dc_queue.h"

#include "server-loop-base.h"
#include "dc_policy.h"

#define DCDBG_READERCONNNAME

/// used for connection processing
/**
 * needs to be externally visible for use in remoteprocessstate.cpp
 */
class ReaderConnMap {
public:
  /// stream state
  int  wInstanceNum;
  int  wWorkNum;
  bool fCheckWorkNum;
  DC_RemoteInStreamCopy  *pInStream;
  DC_RemoteOutStreamCopy *pOutStream;
  /// outstream: set in STREAMCONNECT_ACK, use for recv DATA_ACK
  int  iConnInfo;
  /// instream: set in STREAMCONNECT, use for sending DATA_ACK
  DC_StreamWritePolicy_t eStreamWritePolicy;
#if defined(DCDBG_READERCONNNAME)
  char *sbName;
#endif /* DCDBG_READERCONNNAME */
};

class DC_ReaderServerLoop : public ServerLoop_Base, public ThreadClass {
  DC_FilterService_t *pDC;
public:
  DC_Queue ReaderCmdQ;

public:
  DC_ReaderServerLoop(DC_FilterService_t *pDC_in);

  void *thread_run(void *parg);

  int ClientProcess(int i);
  int MapProcess(int i, enum eMapReason e);

  /// public wrapper to protected base class fcn
  void CloseConn(int i) { ServerLoop_Base::CloseConn(i); };

  int _CloseInstanceConn(int wInstanceNum);

public:
  // thread safe fcns other threads can use to send ReaderCmd to the reader
  int ReaderCmd_AddConn(BaseSocket *psock, InetAddress *paddr);
  int ReaderCmd_EnableConn(int iClient);
  int ReaderCmd_DisableConn(int iClient);
  int ReaderCmd_CloseConn(int iClient);
  int ReaderCmd_CleanupInstance(int wInstanceNum);
};


#endif /* H_DC_THREAD_READER */
// ****************************************************************************
