// ****************************************************************************
//  Name: util.cpp
//  Athr: Michael D. Beynon (mdb)
//  Date: 08/11/2000 : mdb - Created.
//  Desc: DataCutter Filter Service user library - utility routines.
// ****************************************************************************

#include "util.h"
#include "lib/dc_memory.h"

/**
 * look for an entry in the given cfg, and show an error message and
 * optionally exit if not found.
 */
char *FindParm(ConfigInfo &cfg,
	       const char *sbSection, const char *sbName,
	       bool fExitNotFound) {
  char *sbRtn;
  if ((sbRtn = cfg.Find(sbSection, sbName)) == NULL) {
    if (fExitNotFound) {
      cerr << ErrorHeader(wERROR) << "FindParm(): ERROR: "
	   << "no \"[" << sbSection << "] " << sbName << "\" specified."
	   << endl;
      exit(1);  // BUGBUG: we probably should return an error code instead
    }
  }
  return sbRtn;
}

/**
 * convert to an int, and allow a default value if not found
 */
int FindParmIntDefault(ConfigInfo &cfg,
		       const char *sbSection, const char *sbName,
		       int wDefault) {
  char *sb = cfg.Find(sbSection, sbName);
  if (!sb) return wDefault;
  return atoi(sb);
}

/**
 * convert to a bool, and allow a default value if not found
 */
bool FindParmBoolDefault(ConfigInfo &cfg,
			 const char *sbSection, const char *sbName,
			 bool fDefault) {
  char *sb = cfg.Find(sbSection, sbName);
  if (!sb) return fDefault;
  if (StrNCmp(sb, "y", 1) == 0 ||  // yes vs. no
      StrNCmp(sb, "t", 1) == 0 ||  // true vs. false
      StrNCmp(sb, "1", 1) == 0)    // 1 vs. 0
    return true;
  return false;
}

/**
 * These are used to encode 2 ints (say wInstanceNum and wWorkNum)
 * into the wExtra field of a packet.  These need to be combined
 * because the body of the packet is used for the user defined work
 * for APPENDWORK packets.
 */
// BUGBUG: either value > 2 bytes?  We can control wraparound and
// never assign values > 2 bytes (not implemented).
int DC_EncodeInt(int w1, int w2) {
  int wRtn = (w1 << 16) | (w2 & 0xffff);
  return wRtn;
}

void DC_DecodeInt(int wEncoded_in, int *pw1, int *pw2) {
  unsigned int wEncoded = static_cast(unsigned int, wEncoded_in);
  *pw1 = (wEncoded >> 16) & 0xffff;
  *pw2 = (wEncoded & 0xffff);
  return;
}


// ****************************************************************************
