// -*- c++ -*-
// ****************************************************************************
//  Name: workqueue.h
//  Athr: Michael D. Beynon (mdb)
//  Date: 03/03/2001 : mdb - Created.
//  Desc: Part of remote lib process stream abstraction.  Layered above simple
//        queue to support multiple copies of consumers to collectively read
//        work requests from a single queue.
// ****************************************************************************

#if !defined(H_DC_WORKQUEUE)
#define H_DC_WORKQUEUE

#include "lib/dc_mutex.h"
#include "lib/dc_queue.h"
#include "lib/dc_darray.h"
#include "lib/dc_lockedcounter.h"

#include "dc_work.h"
#include "thread-reader.h"


/**
 * collective work queue supporting consumer copies.  The only
 * producer is the reader thread.  The main job is to delay consumers
 * for work k+1 until the last consumer for work k have dequeued.  We
 * also have to worry about too many AppendWork packets blocking the
 * reader thread, by disabling the console connection as needed.
 */
class DC_WorkQueue : public DC_Queue {
public:
  /// enqueue producer side of queue (console process via reader thread)
  int wEnqueueWorkNum;
  /// dequeue consumer side of queue (local filter copies)
  int wDequeueWorkNum;
  int nConsumerDequeue;
  int nTotalConsumers;
  bool       fFirstDequeue;  ///< for handling a blocking dequeue
  DC_Work_t *pworkDequeue;

  DC_Cond cvLocalConsumerReady;  ///< consumer that are too early, or non-first
  DC_Cond cvFirstDequeue;  ///< predicate: fFirstDequeue

  DC_ReaderServerLoop *pSL;
  DC_LockedCounter    *plcFullWorkQs;

public:
  DC_WorkQueue();
  ~DC_WorkQueue();

  // unlocked init fcns
  int setInitialWorkNum(int wWorkNum);
  int setNumConsumers(int nConsumers);
  int setReaderServerLoop(DC_ReaderServerLoop *pSL_in);
  int setCountFullWorkQs(DC_LockedCounter *plcFullWorkQs_in);


  int probe(bool fLock=false);

  int remoteEnqueue(DC_Work_t *pwork, int iConn=-1);

  int localAddConsumer(void);
  int localDequeue(int wWorkNum, DC_Work_t **ppwork, bool fLock=true);
  int localShutdown(void);  ///< drain and shutdown Q
};


#endif /* H_DC_WORKQUEUE */
// ****************************************************************************
